from django.conf.urls.defaults import *
import product
from signals_ahoy.signals import collect_urls

urlpatterns = patterns('product.views',
    (r'^(?P<product_slug>[-\w]+)/$', 
        'get_product', {}, 'satchmo_product'),
    (r'^(?P<product_slug>[-\w]+)/prices/$', 
        'get_price', {}, 'satchmo_product_prices'),
    (r'^(?P<product_slug>[-\w]+)/price_detail/$', 
        'get_price_detail', {}, 'satchmo_product_price_detail'),
)

urlpatterns += patterns('product.views.filters',
    (r'^view/recent/$', 
        'display_recent', {}, 'satchmo_product_recently_added'),
    (r'^view/bestsellers/$', 
        'display_bestsellers', {}, 'satchmo_product_best_selling'),
)


# here we are sending a signal to add patterns to the base of the shop.
# collect_urls.send(sender=product, patterns=urlpatterns, section="product")



from django import template
from satchmo_utils.templatetags import get_filter_args

register = template.Library()

def giftcertificate_order_summary(order):
    """Output a formatted block giving attached gift certifificate details."""
    return {'order' : order}
    
register.inclusion_tag('giftcertificate/_order_summary.html')(giftcertificate_order_summary)



# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from kiwi.component import get_utility

from gazpacho.interfaces import IGazpachoApp

class Context:
    """This class expose the functionality of Gazpacho that plugins are
    allowed to use.

    There is one context object for every project.
    """
    def __init__(self, project):
        # project is private because we don't want plugins to mess up with
        # the cool stuff
        self.__project = project

    def get_application_window(self):
        """This is useful to show dialogs."""
        return get_utility(IGazpachoApp).get_window()

    def get_project(self):
        return self.__project



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""This module defines a Registry class used to store some widget data and
able to perform some queries on that, such as getting a widget's adaptor from
its name or gobject type.

This module also creates a WidgetRegistry instance variable named
widget_registry that can be imported in other modules and used as a
global registry object.
"""

import gobject

from gazpacho.interfaces import BaseWidgetAdaptor

class _Nothing:
    pass

class WidgetRegistry:
    """
    This class is used to store some widget data and able to perform
    some queries on that, such as getting a widget's adaptor from
    its name or gobject type.
    """

    def __init__(self):
        self._widget_adaptors = {}

    def add(self, widget_adaptor):
        """
        Register a widget_adaptor
        @param widget_adaptor: a BaseWidgetAdaptor subclass
        """
        if not isinstance(widget_adaptor, BaseWidgetAdaptor):
            raise TypeError("widget_adaptor must be a BaseWidgetAdaptor "
                            "instance not %r" % type(widget_adaptor))

        # If name is specified, register it two times, hack warning
        if widget_adaptor.name:
            self._widget_adaptors[widget_adaptor.name] = widget_adaptor

        name = gobject.type_name(widget_adaptor.type)
        if name in self._widget_adaptors:
            raise TypeError("%s is already registered" % name)

        self._widget_adaptors[name] = widget_adaptor

    def get_by_name(self, type_name, default=_Nothing):
        """Return widget_adaptor for type_name or None if type_name is not
        registered
        """
        if default is not _Nothing and not self.has_name(type_name):
            return
        return self._widget_adaptors.get(type_name, None)

    def get_by_name_closest(self, type_name):
        """Return widget_adaptor for type_name or closest ancestor"""
        gtype = gobject.type_from_name(type_name)
        while True:
            adapter = self._widget_adaptors.get(gobject.type_name(gtype))
            if adapter is not None:
                return adapter
            gtype = gobject.type_parent(gtype)

    def has_name(self, type_name):
        "True if type_name is registered, False otherwise"
        return type_name in self._widget_adaptors

    def get_by_type(self, obj, default=_Nothing):
        "Return widget_adaptor for a given gobject type gtype"
        if hasattr(obj, '__gtype__'):
            gtype = obj.__gtype__
        else:
            gtype = obj
        type_name = gobject.type_name(gtype)
        return self.get_by_name(type_name, default)

widget_registry = WidgetRegistry()



# Copyright (C) 2005 Johan Dahlin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# TODO:
# Parser tags: atk, relation
# Document public API
# Parser subclass
# Improved unittest coverage
# Old style toolbars
# Require importer/resolver (gazpacho itself)
# GBoxed properties

import os
from gettext import textdomain, dgettext
from xml.parsers import expat

import gobject
import gtk
from gtk import gdk

from gazpacho.loader.custom import adapter_registry, flagsfromstring, \
     str2bool

__all__ = ['ObjectBuilder', 'ParseError']

def open_file(filename):
    if filename.endswith('.gz'):
        import gzip
        return gzip.GzipFile(filename, mode='rb')
    if filename.endswith('.bz2'):
        import bz2
        return bz2.BZ2File(filename, mode='rU')
    return open(filename, mode='rb')

class ParseError(Exception):
    pass

class Stack(list):
    push = list.append
    def peek(self):
        if self:
            return self[-1]

class BaseInfo:
    def __init__(self):
        self.data = ''

    def __repr__(self):
        return '<%s data=%r>' % (self.__class__.__name__,
                                 self.data)

class WidgetInfo(BaseInfo):
    def __init__(self, attrs, parent):
        BaseInfo.__init__(self)
        self.klass = str(attrs.get('class'))
        self.id = str(attrs.get('id'))
        self.constructor  = attrs.get('constructor')
        self.children = []
        self.properties = []
        self.attributes = {} # these are used for layout (e.g. cell renderers)
        self.signals = []
        self.uis = []
        self.accelerators = []
        self.parent = parent
        self.gobj = None
        # If it's a placeholder, used by code for unsupported widgets
        self.placeholder = False
        # these attributes are useful for models:
        self.columns = []
        self.rows = []

    def is_internal_child(self):
        if self.parent and self.parent.internal_child:
            return True
        return False

    def __repr__(self):
        return '<WidgetInfo %s>' % (self.id)

class ChildInfo(BaseInfo):
    def __init__(self, attrs, parent):
        BaseInfo.__init__(self)
        self.type = attrs.get('type')
        self.internal_child = attrs.get('internal-child')
        self.properties = []
        self.packing_properties = []
        self.attributes = {}
        self.placeholder = False
        self.parent = parent
        self.widget = None

    def __repr__(self):
        return '<ChildInfo containing a %s>' % (self.widget)

class PropertyInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.name = str(attrs.get('name'))
        self.translatable = str2bool(attrs.get('translatable', 'no'))
        self.context = str2bool(attrs.get('context', 'no'))
        self.agent = attrs.get('agent') # libglade
        self.comments = attrs.get('comments')

    def __repr__(self):
        return '<PropertyInfo of type %s=%r>' % (self.name, self.data)

class SignalInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.name = attrs.get('name')
        self.handler = attrs.get('handler')
        self.after = str2bool(attrs.get('after', 'no'))
        self.object = attrs.get('object')
        self.last_modification_time = attrs.get('last_modification_time')
        self.gobj = None

class AcceleratorInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.key = gdk.keyval_from_name(attrs.get('key'))
        self.modifiers = flagsfromstring(attrs.get('modifiers'),
                                         flags=gdk.ModifierType)
        self.signal = str(attrs.get('signal'))

class UIInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.id = attrs.get('id')
        self.filename = attrs.get('filename')
        self.merge = str2bool(attrs.get('merge', 'yes'))

class SizeGroupInfo(BaseInfo):
    def __init__(self, name):
        BaseInfo.__init__(self)
        self.name = name
        self.widgets = []

class AttributeInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.name = str(attrs.get('name'))

class ColumnsInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.columns = []

class ColumnInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        type_name = str(attrs.get('type'))
        try:
            self.type = gobject.type_from_name(type_name)
        except RuntimeError:
            raise ParseError('the type %s is not a valid GType' % type_name)

class RowInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        self.cols = []

class ColInfo(BaseInfo):
    def __init__(self, attrs):
        BaseInfo.__init__(self)
        try:
            self.id = int(attrs.get('id'))
        except ValueError:
            raise ParseError('id attribute must be integer')

class ExpatParser(object):
    def __init__(self, builder):
        self._builder = builder
        self.requires = []
        self._stack = Stack()
        self._state_stack = Stack()
        self._ui_state = False
        self._widgets_state = False
        self._sizegroups = []
        self._parser = expat.ParserCreate()
        self._parser.buffer_text = True
        self._parser.StartElementHandler = self._handle_startelement
        self._parser.EndElementHandler = self._handle_endelement
        self._parser.CharacterDataHandler = self._handle_characterdata

    # Public API
    def parse_file(self, filename):
        fp = open_file(filename)
        try:
            self._parser.ParseFile(fp)
        except expat.ExpatError as e:
            raise ParseError(e)

    def parse_stream(self, buf):
        try:
            self._parser.Parse(buf)
        except expat.ExpatError as e:
            raise ParseError(e)

    def _build_ui(self, name, attrs):
        extra = ''
        for attr, value in list(attrs.items()):
            extra += " %s=\"%s\"" % (attr, value)

        ui = self._stack.peek()
        ui.data += "<%s%s>" % (name, extra)

    # Expat callbacks
    def _handle_startelement(self, name, attrs):
        if self._ui_state:
            self._build_ui(name, attrs)
            return
        self._state_stack.push(name)
        name = name.replace('-', '_')
        func = getattr(self, '_start_%s' % name, None)
        if func:
            item = func(attrs)
            self._stack.push(item)

    def _handle_endelement(self, name):
        if self._ui_state:
            ui = self._stack.peek()
            if name != "ui":
                ui.data += "</%s>" % name
                return
            elif not ui.data.startswith("<ui>"):
                ui.data = "<ui>%s</ui>" % ui.data

        self._state_stack.pop()
        name = name.replace('-', '_')
        func = getattr(self, '_end_%s' % name, None)
        if func:
            item = self._stack.pop()
            func(item)

    def _handle_characterdata(self, data):
        info = self._stack.peek()
        if info:
            info.data += str(data)

    # Tags
    def _start_glade_interface(self, attrs):
        # libglade extension, add a domain argument to the interface
        if 'domain' in attrs:
            self._builder._domain = str(attrs['domain'])
        self._builder._version = "libglade"

    def _end_glade_interface(self, obj):
        pass

    def _start_interface(self, attrs):
        # libglade extension, add a domain argument to the interface
        if 'domain' in attrs:
            self._builder._domain = str(attrs['domain'])
        self._builder._version = "gtkbuilder"

    def _end_interface(self, attrs):
        pass

    def _start_requires(self, attrs):
        self.requires.append(attrs)

    def _end_requires(self, obj):
        pass

    def _start_signal(self, attrs):
        if not 'name' in attrs:
            raise ParseError("<signal> needs a name attribute")
        if not 'handler' in attrs:
            raise ParseError("<signal> needs a handler attribute")
        return SignalInfo(attrs)

    def _end_signal(self, signal):
        obj = self._stack.peek()
        obj.signals.append(signal)

    def _start__common(self, attrs):
        if not 'class' in attrs:
            raise ParseError("<widget> needs a class attribute")
        if not 'id' in attrs:
            raise ParseError("<widget> needs an id attribute")

        return WidgetInfo(attrs, self._stack.peek())

    def _start_widget(self, attrs):
        if self._widgets_state:
            obj = self._stack.peek()
            obj.widgets.append(str(attrs.get('name')))
            return
        else:
            if self._builder._version == 'gtkbuilder':
                raise ParseError("Unknown tag: widget")
        return self._start__common(attrs)

    def _start_object(self, attrs):
        if self._builder._version == 'libglade':
            raise ParseError("Unknown tag: object")
        return self._start__common(attrs)

    def _end__common(self, obj):
        obj.parent = self._stack.peek()

        if not obj.gobj:
            obj.gobj = self._builder._build_phase1(obj)

        self._builder._build_phase2(obj)

        if obj.parent:
            obj.parent.widget = obj.gobj

    def _end_widget(self, attrs):
        if self._widgets_state:
            return
        return self._end__common(attrs)

    def _end_object(self, attrs):
        return self._end__common(attrs)

    def _start_child(self, attrs):
        obj = self._stack.peek()
        if not obj.gobj:
            obj.gobj = self._builder._build_phase1(obj)

        return ChildInfo(attrs, parent=obj)

    def _end_child(self, child):
        obj = self._stack.peek()
        obj.children.append(child)

    def _start_property(self, attrs):
        if not 'name' in attrs:
            raise ParseError("<property> needs a name attribute")
        return PropertyInfo(attrs)

    def _end_property(self, prop):
        if prop.agent and prop.agent not in ('libglade', 'gazpacho'):
            return

        # gettext cannot really handle empty strings, so we need to filter
        # them out, otherwise we'll get the po header as the content!
        # Note that we should not write properties with empty strings from
        # the start, but that is another bug
        if prop.translatable and prop.data:
            if not self._builder._ignore_domain:
                prop.data = dgettext(self._builder._domain, prop.data)

        obj = self._stack.peek()

        property_type = self._state_stack.peek()
        if property_type == 'widget' or property_type == 'object':
            obj.properties.append(prop)
        elif property_type == 'packing':
            obj.packing_properties.append(prop)
        else:
            raise ParseError("property must be a node of widget or packing")

    def _start_ui(self, attrs):
        if self._builder._version != 'gtkbuilder':
            if not 'id' in attrs:
                raise ParseError("<ui> needs an id attribute")
        self._ui_state = True
        return UIInfo(attrs)

    def _end_ui(self, ui):
        self._ui_state = False
        if not ui.data or ui.filename:
            raise ParseError("<ui> needs CDATA or filename")

        obj = self._stack.peek()
        obj.uis.append(ui)

    def _start_placeholder(self, attrs):
        pass

    def _end_placeholder(self, placeholder):
        obj = self._stack.peek()
        obj.placeholder = True

    def _start_accelerator(self, attrs):
        if not 'key' in attrs:
            raise ParseError("<accelerator> needs a key attribute")
        if not 'modifiers' in attrs:
            raise ParseError("<accelerator> needs a modifiers attribute")
        if not 'signal' in attrs:
            raise ParseError("<accelerator> needs a signal attribute")
        return AcceleratorInfo(attrs)

    def _end_accelerator(self, accelerator):
        obj = self._stack.peek()
        obj.accelerators.append(accelerator)

    def _start_widgets(self, attrs):
        self._widgets_state = True
        obj = self._stack.peek()
        return SizeGroupInfo(obj.id)

    def _end_widgets(self, obj):
        self._builder._add_sizegroup_widgets(obj.name, obj.widgets)

    def _start_attribute(self, attrs):
        if not 'name' in attrs:
            raise ParseError("<attribute> needs a name attribute")
        return AttributeInfo(attrs)

    def _end_attribute(self, attribute):
        obj = self._stack.peek()

        try:
            data = int(attribute.data)
        except ValueError:
            raise ParseError("attribute value must be an integer")

        property_type = self._state_stack.peek()
        if property_type == 'attributes':
            obj.attributes[attribute.name] = data
        else:
            raise ParseError("attribute must be a node of <attributes>")

    def _start_column(self, attrs):
        if not 'type' in attrs:
            raise ParseError("<column> needs a type attribute")
        return ColumnInfo(attrs)

    def _end_column(self, column):
        obj = self._stack.peek()
        parent_type = self._state_stack.peek()
        if parent_type == 'columns':
            obj.columns.append(column)
        else:
            raise ParseError("column must be a node of <columns>")

    def _start_row(self, attrs):
        return RowInfo(attrs)

    def _end_row(self, row):
        obj = self._stack.peek()
        parent_type = self._state_stack.peek()
        if parent_type == 'data':
            obj.rows.append(row)
        else:
            raise ParseError("row must be a node of <data>")

    def _start_col(self, attrs):
        if not 'id' in attrs:
            raise ParseError("<col> needs an id attribute")
        return ColInfo(attrs)

    def _end_col(self, col):
        obj = self._stack.peek()
        parent_type = self._state_stack.peek()
        if parent_type == 'row':
            obj.cols.append(col)
        else:
            raise ParseError("col must be a node of <row>")

class ObjectBuilder:
    def __init__(self, filename='', buffer=None, root=None, placeholder=None,
                 custom=None, domain=None, ignore_domain=False):
        if ((not filename and not buffer) or
            (filename and buffer)):
            raise TypeError("need a filename or a buffer")

        self._filename = filename
        self._buffer = buffer
        self._root = root
        self._placeholder = placeholder
        self._custom = custom
        self._domain = domain
        self._ignore_domain = ignore_domain

        # maps the classes of widgets that failed to load to lists of
        # widget ids
        self._unsupported_widgets = {}

        # name -> GObject
        self._widgets = {}
        self._signals = []
        # GObject -> Constructor
        self._constructed_objects = {}
        # ui definition name -> UIMerge, see _mergeui
        self._uidefs = {}
        # ui definition name -> constructor name (backwards compatibility)
        self._uistates = {}
        self._tooltips = gtk.Tooltip()
        #self._tooltips.enable()
        self._focus_widget = None
        self._default_widget = None
        self._toplevel = None
        self._accel_group = None
        self._delayed_properties = {}
        self._internal_children = {}
        self._sizegroup_widgets = {}

        # Public
        self.toplevels = []
        self.sizegroups = []

        # If domain is not specified, fetch the default one by
        # calling textdomain() without arguments
        if not domain:
            domain = textdomain()
            # messages is the default domain, ignore it
            if domain == 'messages':
                domain = None

        self._parser = ExpatParser(self)
        if filename:
            self._parser.parse_file(filename)
        elif buffer:
            self._parser.parse_stream(buffer)
        self._parse_done()

    def __len__(self):
        return len(self._widgets)

    def __bool__(self):
        return True

    # Public API

    def get_domain(self):
        return self._domain

    def get_version(self):
        return self._version

    def get_widget(self, widget):
        return self._widgets.get(widget)

    def get_widgets(self):
        return list(self._widgets.values())

    def get_unsupported_widgets(self):
        """
        Get a dict of the widgets that could not be loaded because
        they're not supported. The dict maps the class of the widget
        to a list of widget ids.

        @return: the unsupported widgets
        @rtype: dict
        """
        return self._unsupported_widgets

    def signal_autoconnect(self, obj):
        for gobj, name, handler_name, after, object_name in self.get_signals():
            # Firstly, try to map it as a dictionary
            try:
                handler = obj.get(handler_name)
            except (AttributeError, TypeError):
                # If it fails, try to map it to an attribute
                handler = getattr(obj, handler_name, None)
            if not handler:
                continue

            if object_name:
                other = self._widgets.get(object_name)
                if after:
                    gobj.connect_object_after(name, handler, other)
                else:
                    gobj.connect_object(name, handler, other)
            else:
                if after:
                    gobj.connect_after(name, handler)
                else:
                    gobj.connect(name, handler)

    def show_windows(self):
# Doesn't quite work, disabled for now
#         # First set focus, warn if more than one is focused
#         toplevel_focus_widgets = []
#         for widget in self.get_widgets():
#             if not isinstance(widget, gtk.Widget):
#                 continue

#             if widget.get_data('gazpacho::is-focus'):
#                 toplevel = widget.get_toplevel()
#                 name = toplevel.get_name()
#                 if name in toplevel_focus_widgets:
#                     print ("Warning: Window %s has more than one "
#                            "focused widget" % name)
#                 toplevel_focus_widgets.append(name)

        # At last, display all of the visible windows
        for toplevel in self.toplevels:
            if not isinstance(toplevel, gtk.Window):
                continue
            value = toplevel.get_data('gazpacho::visible')
            toplevel.set_property('visible', value)

    def get_internal_child(self, gobj):
        if not gobj in self._internal_children:
            return []
        return self._internal_children[gobj]

    def get_internal_children(self):
        return self._internal_children

    # Adapter API

    def add_signal(self, gobj, name, handler, after=False, sig_object=None):
        self._signals.append((gobj, name, handler, after, sig_object))

    def get_signals(self):
        return self._signals

    def find_resource(self, filename):
        dirname = os.path.dirname(self._filename)
        path = os.path.join(dirname, filename)
        if os.access(path, os.R_OK):
            return path

    def get_ui_definitions(self):
        return [(name, info.data, info.merge_id)
                    for name, info in list(self._uidefs.items())]

    def get_constructor(self, gobj):
        return self._constructed_objects[gobj]

    def ensure_accel(self):
        if not self._accel_group:
            self._accel_group = gtk.AccelGroup()
            if self._toplevel:
                self._toplevel.add_accel_group(self._accel_group)
        return self._accel_group

    def add_delayed_property(self, obj_id, pspec, value):
        self._delayed_properties.setdefault(obj_id, []).append((pspec, value))

    # Private

    def _setup_signals(self, gobj, signals):
        for signal in signals:
            self.add_signal(gobj, signal.name, signal.handler,
                           signal.after, signal.object)

    def _setup_accelerators(self, widget, accelerators):
        if not accelerators:
            return

        accel_group = self.ensure_accel()
        widget.set_data('gazpacho::accel-group', accel_group)
        for accelerator in accelerators:
            widget.add_accelerator(accelerator.signal,
                                   accel_group,
                                   accelerator.key,
                                   accelerator.modifiers,
                                   gtk.ACCEL_VISIBLE)

    def _apply_delayed_properties(self):
        for obj_id, props in list(self._delayed_properties.items()):
            widget = self._widgets.get(obj_id)
            if widget is None:
                raise AssertionError

            type_name = gobject.type_name(widget)
            adapter = adapter_registry.get_adapter(widget, type_name, self)

            prop_list = []
            for pspec, value in props:
                if gobject.type_is_a(pspec.value_type, gobject.GObject):
                    other = self._widgets.get(value)
                    if other is None:
                        raise ParseError(
                            "property %s of %s refers to object %s which "
                            "does not exist" % (pspec.name, obj_id,value))
                    prop_list.append((pspec.name, other))
                else:
                    raise NotImplementedError(
                        "Only delayed object properties are "
                        "currently supported")

            adapter.set_properties(widget, prop_list)

    def _merge_ui(self, uimanager_name, name,
                  filename='', data=None, merge=True):
        #uimanager = self._widgets[uimanager_name]
        uimanager = gtk.UIManager()
        if merge:
            if filename:
                filename = self.find_resource(filename)
                # XXX Catch GError
                merge_id = uimanager.add_ui_from_file(filename)
            elif data:
                # XXX Catch GError
                merge_id = uimanager.add_ui_from_string(data)
            else:
                raise AssertionError
        else:
            merge_id = -1

        class UIMerge:
            def __init__(self, uimanager, filename, data, merge_id):
                self.uimanager = uimanager,
                self.filename = filename
                self.data = data
                self.merge_id = merge_id

        current = self._uidefs.get(name)
        if current:
            current.merge_id = merge_id
        else:
            self._uidefs[name] = UIMerge(uimanager, filename, data,
                                         merge_id)

        # Backwards compatibility
        self._uistates[name] = uimanager_name

    def _uimanager_construct(self, uimanager_name, obj_id):
        uimanager = gtk.UIManager() #self._widgets[uimanager_name]
        widget = uimanager.get_widget('ui/' + obj_id)
        if widget is not None:
            setattr(widget, 'uimanagername', uimanager_name)
        else:
            # XXX: untested code
            uimanager_name = self._uistates.get(obj_id)
            if not uimanager_name:
                #raise AssertionError
                return
            uimanager = self._widgets[uimanager_name]

        return widget

    def _find_internal_child(self, obj):
        child = None
        childname = str(obj.parent.internal_child)
        parent = obj.parent
        while parent:
            if isinstance(parent, ChildInfo):
                parent = parent.parent
                continue

            gparent = parent.gobj
            if not gparent:
                break

            adapter = adapter_registry.get_adapter(gparent, parent.klass, self)
            child = adapter.find_internal_child(gparent, childname)
            if child is not None:
                break

            parent = parent.parent

        if child is not None:
            children = self._internal_children.setdefault(gparent, [])
            children.append((childname, child))
            # Childname is attribute name, which we use to find the child
            # in the parent widget.
            # obj.id stores the real name, it needs to be set manually here
            # since we're not calling the widget adapters constructor
            child.set_name(obj.id)
            child.set_data('gazpacho::internal-child-name', childname)
        return child

    def _create_custom(self, obj):
        kwargs = dict(name=obj.id)
        for prop in obj.properties:
            prop_name = prop.name
            if prop_name in ('string1', 'string2',
                             'creation_function',
                             'last_modification_time'):
                kwargs[prop_name] = prop.data
            elif prop_name in ('int1', 'int2'):
                kwargs[prop_name] = int(prop.data)

        if not self._custom:
            return gtk.Label('<Custom: %s>' % obj.id)
        elif callable(self._custom):
            func = self._custom
            return func(**kwargs)
        else:
            func_name = kwargs['creation_function']
            try:
                func = self._custom[func_name]
            except (TypeError, KeyError, AttributeError):
                func = getattr(self._custom, func_name, None)

            return func(name=obj.id,
                        string1=kwargs.get('string1', None),
                        string2=kwargs.get('string2', None),
                        int1=kwargs.get('int1', None),
                        int2=kwargs.get('int2', None))

    def _create_placeholder(self, obj=None):
        if not self._placeholder:
            return

        if not obj:
            name = 'unknown'
        else:
            name = obj.id

        return self._placeholder(name)

    def _add_widget(self, object_id, gobj):
        #gobj.set_data('gazpacho.objectid', object_id)
        setattr(gobj, 'objectid', object_id)
        self._widgets[object_id] = gobj.objectid

    def _build_phase1(self, obj):
        assert obj.gobj is None, obj.id

        root = self._root
        if root and root != obj.id:
            return

        if obj.klass == 'Custom':
            gobj = self._create_custom(obj)
            if gobj:
                self._add_widget(obj.id, gobj)
            return gobj

        try:
            gtype = gobject.type_from_name(obj.klass)
        except RuntimeError:
            self._unsupported_widgets.setdefault(obj.klass, []).append(obj.id)
            obj.placeholder = True
            return self._create_placeholder(obj)

        adapter = adapter_registry.get_adapter(gtype, obj.klass, self)

        construct, normal = adapter.get_properties(gtype,
                                                   obj.id,
                                                   obj.properties)
        if obj.is_internal_child():
            gobj = self._find_internal_child(obj)
        elif obj.constructor:
            if obj.constructor in self._widgets:
                gobj = self._uimanager_construct(obj.constructor, obj.id)
                constructor = obj.constructor
            # Backwards compatibility
            elif obj.constructor in self._uistates:
                constructor = self._uistates[obj.constructor]
                gobj = self._uimanager_construct(constructor, obj.id)
            else:
                raise ParseError("constructor %s for object %s could not "
                                 "be found" % (obj.constructor, obj.id))
            self._constructed_objects[gobj] = self._widgets[constructor]
        else:
            gobj = adapter.construct(obj.id, gtype, construct)

        if gobj:
            self._add_widget(obj.id, gobj)

            if isinstance(gobj, gtk.ListStore):
                adapter.set_column_types(gobj, obj.columns)
                adapter.fill(gobj, obj.rows)

            adapter.set_properties(gobj, normal)

            # This is a little tricky
            # We assume the default values for all these are nonzero, eg
            # either False or None
            # We also need to handle the case when we have two labels, if we
            # do we respect the first one. This is due to a bug in the save code
            for propinfo in obj.properties:
                key = 'i18n_is_translatable_%s' % propinfo.name
                if not (hasattr(gobj, key) and propinfo.translatable):
                    setattr(gobj, key, propinfo.translatable)

                key = 'i18n_has_context_%s' % propinfo.name
                if not hasattr(gobj, key) and propinfo.context:
                    setattr(gobj, key, propinfo.context)

                # XXX: Rename to i18n_comments
                key = 'i18n_comment_%s' % propinfo.name
                if not hasattr(gobj, key) and propinfo.comments:
                    setattr(gobj, key, propinfo.comments)

        return gobj

    def _build_phase2(self, obj):
        # If we have a root set, we don't want to construct all
        # widgets, filter out unwanted here
        root = self._root
        if root and root != obj.id:
            return

        # Skip this step for placeholders, so we don't
        # accidentally try to pack something into unsupported widgets
        if obj.placeholder:
            return

        gobj = obj.gobj
        if not gobj:
            return

        adapter = adapter_registry.get_adapter(gobj, obj.klass, self)

        for child in obj.children:
            self._pack_child(adapter, gobj, child)

        self._setup_signals(gobj, obj.signals)
        self._setup_accelerators(gobj, obj.accelerators)

        # Toplevels
        if not obj.parent:
            if isinstance(gobj, gtk.UIManager):
                for ui in obj.uis:
                    self._merge_ui(obj.id,
                                   ui.id, ui.filename, ui.data, ui.merge)
                    self.accelgroup = gobj.get_accel_group()
            elif isinstance(gobj, gtk.Window):
                self._set_toplevel(gobj)
            self.toplevels.append(gobj)

    def _pack_child(self, adapter, gobj, child):

        if child.placeholder:
            widget = self._create_placeholder()
            if not widget:
                return
        elif child.widget:
            widget = child.widget
        else:
            return

        if child.internal_child:
            gobj = child.parent.gobj
            name = child.parent.id
            if isinstance(gobj, gtk.Widget):
                gobj.set_name(name)
            self._add_widget(name, gobj)
            return

        # 5) add child
        try:
            adapter.add(gobj,
                        widget,
                        child.packing_properties,
                        child.type)
        except NotImplementedError:
            raise ParseError(
                '%s of type %s has children, but its not supported' % (
                child.parent.id,
                gobject.type_name(gobj)))

        if isinstance(gobj, gtk.CellLayout):
            adapter.set_cell_renderer_attributes(gobj, widget, child.attributes)

    def _attach_accel_groups(self):
        # This iterates of all objects constructed by a gtk.UIManager
        # And attaches an accelgroup to the toplevel window of them
        for widget, constructor in list(self._constructed_objects.items()):
            if not isinstance(constructor, gtk.UIManager):
                continue
            toplevel = widget.get_toplevel()
            if not isinstance(toplevel, gtk.Window):
                continue
            accel_group = constructor.get_accel_group()
            if not accel_group in gtk.accel_groups_from_object(toplevel):
                toplevel.add_accel_group(accel_group)

    def _add_sizegroup_widgets(self, group_name, widget_names):
        self._sizegroup_widgets[group_name] = widget_names

    def _setup_sizegroups(self):
        # gtkbuilder way
        for group_name, widget_names in list(self._sizegroup_widgets.items()):
            for widget_name in widget_names:
                widget = self._widgets[widget_name]
                widget.set_data('gazpacho::sizegroup', group_name)

        for widget in list(self._widgets.values()):
            # Collect all the sizegroups
            if isinstance(widget, gtk.SizeGroup):
                self.sizegroups.append(widget)
                continue

            # And setup the widgets which has a sizegroup
            if not isinstance(widget, gtk.Widget):
                continue

            group_name = widget.get_data('gazpacho::sizegroup')
            if group_name is None:
                continue
            group = self.get_widget(group_name)
            if group is None:
                raise ParseError("sizegroup %s does not exist" %
                                 group_name)
            group.add_widget(widget)

            # Keep a list of widgets inside the sizegroup.
            # Perhaps GTK+ should provide an api for this.
            sgwidgets = group.get_data('gazpacho::sizegroup-widgets')
            if sgwidgets is None:
                sgwidgets = []
                group.set_data('gazpacho::sizegroup-widgets', sgwidgets)
            sgwidgets.append(widget)

    def _parse_done(self):
        self._apply_delayed_properties()
        self._attach_accel_groups()
        self._setup_sizegroups()
        self.show_windows()

    def _set_toplevel(self, window):
        if self._focus_widget:
            self._focus_widget.grab_focus()
            self._focus_widget = None
        if self._default_widget:
            if self._default_widget.flags() & gtk.CAN_DEFAULT:
                self._default_widget.grab_default()
            self._default_widget = None
        if self._accel_group:
            self._accel_group = None

        # the window should hold a reference to the tooltips object
        window.set_data('gazpacho::tooltips', self._tooltips)
        self._toplevel = window

if __name__ == '__main__':
    import sys
    ob = ObjectBuilder(filename=sys.argv[1])
    for toplevel in ob.toplevels:
        if not isinstance(toplevel, gtk.Window):
            continue
        toplevel.connect('delete-event', gtk.main_quit)
        toplevel.show_all()

    gtk.main()



from decimal import Decimal
from django.contrib.auth.models import Group
from django.db import models
from django.utils.translation import ugettext_lazy as _
from product import signals
from product.models import Product, Price, PriceAdjustment, PriceAdjustmentCalc
from satchmo_utils.fields import CurrencyField
from threaded_multihost import threadlocals
import datetime
import logging

log = logging.getLogger('tieredpricing.models')

class PricingTierManager(models.Manager):
    
    def by_user(self, user):
        """Get the pricing tiers for a user"""
        key = 'TIER_%i' % user.id
        current = threadlocals.get_thread_variable(key)
        
        if current is None:
            groups = user.groups.all()
            if groups:            
                q = self.all()
                for group in groups:
                    q = q.filter(group=group)
                if q.count() > 0:
                    current = list(q)
                    
            if current is None:
                current = "no"
                
            threadlocals.set_thread_variable(key, current)

        if current == "no":
            raise PricingTier.DoesNotExist
        
        return current

class PricingTier(models.Model):
    """A specific pricing tier, such as "trade customers"
    """
    group = models.OneToOneField(Group, help_text=_('The user group that will receive the discount'))
    title = models.CharField(_('Title'), max_length=50)
    discount_percent = models.DecimalField(_("Discount Percent"), null=True, blank=True,
        max_digits=5, decimal_places=2,
        help_text=_("This is the discount that will be applied to every product if no explicit Tiered Price exists for that product.  Leave as 0 if you don't want any automatic discount in that case."))
        
    objects = PricingTierManager()
        
    def __unicode__(self):
        return self.title
    
class TieredPriceManager(models.Manager):
    
    def by_product_qty(self, tier, product, qty=Decimal('1')):
        """Get the tiered price for the specified product and quantity."""
        
        qty_discounts = product.tieredprices.exclude(expires__isnull=False, expires__lt=datetime.date.today()).filter(quantity__lte=qty, pricingtier=tier)
        
        if qty_discounts.count() > 0:
            # Get the price with the quantity closest to the one specified without going over
            return qty_discounts.order_by('-quantity')[0]
        raise TieredPrice.DoesNotExist

class TieredPrice(models.Model):
    """
    A Price which applies only to special tiers.
    """
    pricingtier = models.ForeignKey(PricingTier, related_name="tieredprices")
    product = models.ForeignKey(Product, related_name="tieredprices")
    price = CurrencyField(_("Price"), max_digits=14, decimal_places=6, )
    quantity = models.DecimalField(_("Discount Quantity"), max_digits=18, decimal_places=6,  default='1', help_text=_("Use this price only for this quantity or higher"))
    expires = models.DateField(_("Expires"), null=True, blank=True)
    
    objects = TieredPriceManager()
    
    def _dynamic_price(self):
        """Get the current price as modified by all listeners."""
        adjust = PriceAdjustmentCalc(self)
        signals.satchmo_price_query.send(self, adjustment=adjust,
            slug=self.product.slug, discountable=self.product.is_discountable)
        return adjust.final_price()

    dynamic_price = property(fget=_dynamic_price)
    
    def save(self, **kwargs):
        prices = TieredPrice.objects.filter(product=self.product, quantity=self.quantity)
        if self.expires:
            prices = prices.filter(expires=self.expires)
        else:
            prices = prices.filter(expires__isnull=True)
        if self.id:
            prices = prices.exclude(id=self.id)
        if prices.count():
            return #Duplicate Price

        super(TieredPrice, self).save(**kwargs)

    class Meta:
        ordering = ['pricingtier', 'expires', '-quantity']
        verbose_name = _("Tiered Price")
        verbose_name_plural = _("Tiered Prices")
        unique_together = (("pricingtier", "product", "quantity", "expires"),)
        
def tiered_price_listener(signal, adjustment=None, **kwargs):
    """Listens for satchmo_price_query signals, and returns a tiered price instead of the
    default price.  

    Requires threaded_multihost.ThreadLocalMiddleware to be installed so
    that it can determine the current user."""
    
    if kwargs.has_key('discountable'):
        discountable = kwargs['discountable']
    else:
        discountable = adjustment.product.is_discountable
    
    if discountable:
        product = adjustment.product
        user = threadlocals.get_current_user()
        if user and not user.is_anonymous():
            try:
                tiers = PricingTier.objects.by_user(user)
                log.debug('got tiers: %s', tiers)
                best = None
                besttier = None
                currentprice = adjustment.final_price()
                qty = adjustment.price.quantity
                for tier in tiers:
                    candidate = None
                    try:
                        tp = TieredPrice.objects.by_product_qty(tier, product, qty)
                        log.debug("Found a Tiered Price for %s qty %d = %s", product.slug, qty, tp.price)
                        candidate = tp.price
                    except TieredPrice.DoesNotExist:
                        pcnt = tier.discount_percent
                        if pcnt is not None and pcnt != 0:
                            candidate = currentprice * (100-pcnt)/100
                        
                    if best is None or (candidate and candidate < best):
                        best = candidate
                        besttier = tier
                    
                        log.debug('best = %s', best)
                
                if best is not None:
                    delta = currentprice - best
                    adjustment += PriceAdjustment(
                        'tieredpricing', 
                        _('Tiered Pricing for %(tier)s' % { 'tier': besttier.group.name}), 
                        delta)
        
            except PricingTier.DoesNotExist:
                pass

signals.satchmo_price_query.connect(tiered_price_listener)



#!/usr/bin/env python
# -*- encoding: UTF-8 -*-
#import fastcgi
import mainapp # global_conf setup
#import logging
from notmm.controllers.wsgi import WSGIController
from utils import make_app
from gevent_fastcgi.server import FastCGIServer
from gevent_fastcgi.wsgi import WSGIRequestHandler
#log = logging.getLogger('notmm.controllers.wsgi')

def initmain(WSGIHandlerClass=WSGIController):

    
    # check for i18n 
    #if bool(mainapp.global_conf['mainapp']['i18n.enable'] == 'true'):
    #    try:
    #        from notmm.controllers.i18n import I18NController as WSGIHandlerClass
    #    except ImportError:
    #        log.debug('generic i18n support is disabled')
    
    wsgi_app = make_app(handlerClass=WSGIHandlerClass)

    # fastcgi
    #import pdb; pdb.set_trace()
    #s = fastcgi.ForkingWSGIServer(wsgi_app, workers=1)
    #s.serve_forever()
    handler = WSGIRequestHandler(wsgi_app)
    s = FastCGIServer(('127.0.0.1', 8808), handler)
    s.serve_forever()
    #return wsgi_app

if __name__ ==  '__main__':
    initmain()



from django.core.management.base import NoArgsCommand
from django.core import urlresolvers
import sys
import django
from decimal import Decimal
import types

class Command(NoArgsCommand):
    help = "Check the system to see if the Satchmo components are installed correctly."
    
    def handle_noargs(self, **options):
        from django.conf import settings
        errors = []
        print "Checking your satchmo configuration."
        try:
            import satchmo_store
        except ImportError:
            errors.append("Satchmo is not installed correctly. Please verify satchmo is on your sys path.")
        try:
            import notmm
        except ImportError:
            errors.append("The notmm toolkit was not found.")
        
        print "Using Django version %s" % django.get_version()
        print "Using Satchmo version %s" % satchmo_store.VERSION # TODO: replace with livestore.VERSION
        print "Detected notmm toolkit %s" % notmm.api_version 
        
        # Try importing all our dependencies
        try:
            import Crypto.Cipher
        except ImportError:
            errors.append("The Python Cryptography Toolkit is not installed.")
        try:
            import Image
        except ImportError:
            try:
                import PIL as Image
            except ImportError:
                errors.append("The Python Imaging Library is not installed.")
        try:
            import reportlab
        except ImportError:
            errors.append("Reportlab is not installed.")
        try:
            import trml2pdf
        except ImportError:
            errors.append("Tiny RML2PDF is not installed.")
        try:
            import registration
        except ImportError:
            errors.append("Django registration is not installed.")
        try:
            import yaml
        except ImportError:
            errors.append("YAML is not installed.")
        try:
            import sorl
        except ImportError:
            errors.append("Sorl imaging library is not installed.")
        try:
            import app_plugins
        except ImportError:
            errors.append("App plugins is not installed.")
        try:
            import livesettings
        except ImportError:
            errors.append("Livesettings is not installed.")
        try:
            import keyedcache
        except ImportError:
            errors.append("Keyedcache is not installed.")
        try:
            cache_avail = settings.CACHE_BACKEND
        except AttributeError:
            errors.append("A CACHE_BACKEND must be configured.")
        # Try looking up a url to see if there's a misconfiguration there    
        try:
            url = urlresolvers.reverse('satchmo_search')
        except Exception, e:
            errors.append("Unable to resolve url. Received error- %s" % e)
        from l10n.l10n_settings import get_l10n_default_currency_symbol
        if not isinstance(get_l10n_default_currency_symbol(),types.UnicodeType):
            errors.append("Your currency symbol should be a unicode string.")
        if 'satchmo_store.shop.SSLMiddleware.SSLRedirect' not in settings.MIDDLEWARE_CLASSES:
            errors.append("You must have satchmo_store.shop.SSLMiddleware.SSLRedirect in your MIDDLEWARE_CLASSES.")
        if 'satchmo_store.shop.context_processors.settings' not in settings.TEMPLATE_CONTEXT_PROCESSORS:
            errors.append("You must have satchmo_store.shop.context_processors.settings in your TEMPLATE_CONTEXT_PROCESSORS.")
        if 'threaded_multihost.middleware.ThreadLocalMiddleware' not in settings.MIDDLEWARE_CLASSES:
            errors.append("You must install django threaded multihost \n and place threaded_multihost.middleware.ThreadLocalMiddleware in your MIDDLEWARE_CLASSES.")
        if 'satchmo_store.accounts.email-auth.EmailBackend' not in settings.AUTHENTICATION_BACKENDS:
            errors.append("You must have satchmo_store.accounts.email-auth.EmailBackend in your AUTHENTICATION_BACKENDS")
        if len(settings.SECRET_KEY) == 0:
            errors.append("You must have SECRET_KEY set to a valid string in your settings.py file")
        python_ver = Decimal("%s.%s" % (sys.version_info[0], sys.version_info[1]))
        if python_ver < Decimal("2.4"):
            errors.append("Python version must be at least 2.4.")
        if python_ver < Decimal("2.5"):
            try:
                from elementtree.ElementTree import Element
            except ImportError:
                errors.append("Elementtree is not installed.")
        if len(errors) == 0:
            print "Your configuration has no errors."
        else:
            print "The following errors were found:"
            for error in errors:
                print error



from notmm.utils.urlmap import RegexURLMap, url

urlpatterns = RegexURLMap(label='Product category urlconf')
urlpatterns.add_routes('product.views',
    url(r'^(?P<parent_slugs>([-\w]+/)*)(?P<slug>[-\w]+)/$', 
        'category_view', {}, name='satchmo_category'),
    url(r'^$', 'category_index', {}, name='satchmo_category_index'),
)



from setuptools import setup, find_packages

setup(name="Elixir",
      version="0.7.1",
      description="Declarative Mapper for SQLAlchemy",
      long_description="""
Elixir
======

A declarative layer on top of SQLAlchemy. It is a fairly thin wrapper, which
provides the ability to create simple Python classes that map directly to
relational database tables (this pattern is often referred to as the Active
Record design pattern), providing many of the benefits of traditional
databases without losing the convenience of Python objects.

Elixir is intended to replace the ActiveMapper SQLAlchemy extension, and the
TurboEntity project but does not intend to replace SQLAlchemy's core features,
and instead focuses on providing a simpler syntax for defining model objects
when you do not need the full expressiveness of SQLAlchemy's manual mapper
definitions.

SVN version: <http://elixir.ematia.de/svn/elixir/trunk#egg=Elixir-dev>
""",
      author="Gaetan de Menten, Daniel Haus and Jonathan LaCour",
      author_email="sqlelixir@googlegroups.com",
      url="http://elixir.ematia.de",
      license = "MIT License",
      install_requires = [
          "SQLAlchemy >= 0.4.0"
      ],
      packages=find_packages(exclude=['ez_setup', 'tests', 'examples']),
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Topic :: Database :: Front-Ends",
          "Topic :: Software Development :: Libraries :: Python Modules"
      ],
      test_suite = 'nose.collector')



"""Protx checkout custom views"""

from django import http
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from livesettings import config_get_group
from payment.views import payship, confirm
import logging
from satchmo_utils.dynamic import lookup_template

log = logging.getLogger('protx.views')
    
def pay_ship_info(request):
    return payship.credit_pay_ship_info(request, 
            config_get_group('PAYMENT_PROTX'),
            template="shop/checkout/protx/pay_ship.html")
    
def confirm_info(request, template='shop/checkout/protx/confirm.html', extra_context={}):
    payment_module = config_get_group('PAYMENT_PROTX')
    controller = confirm.ConfirmController(request, payment_module)
    controller.templates['CONFIRM'] = template
    controller.extra_context = extra_context
    controller.onForm = secure3d_form_handler
    controller.confirm()
    return controller.response
            
def confirm_secure3d(request, secure3d_template='shop/checkout/secure3d_form.html', 
    confirm_template='shop/checkout/confirm.html', extra_context={}):
    """Handles confirming an order and processing the charges when secured by secure3d.
 
    """
    payment_module = config_get_group('PAYMENT_PROTX')
    controller = confirm.ConfirmController(request, payment_module, extra_context=extra_context)
    controller.template['CONFIRM'] = confirm_template
    if not controller.sanity_check():
        return controller.response
    
    auth3d = request.session.get('3D', None)
    if not auth3d:
        controller.processorMessage = _('3D Secure transaction expired. Please try again.')

    else:
        if request.method == "POST":
            returnMD = request.POST.get('MD', None)
            if not returnMD:
                template = lookup_template(payment_module, secure3d_template)
                ctx = RequestContext(request, {'order': controller.order, 'auth': auth3d })
                return render_to_response(template, context_instance=ctx)

            elif returnMD == auth3d['MD']:
                pares = request.POST.get('PaRes', None)
                controller.processor.prepare_data(controller.order)
                controller.processor.prepare_data3d(returnMD, pares)
                if controller.process():
                    return controller.onSuccess(controller)
                else:
                    controller.processorMessage = _('3D Secure transaction was not approved by payment gateway. Please contact us.')
        else:
            template = lookup_template(payment_module, secure3d_template)
            ctx =RequestContext(request, {
                'order': controller.order, 'auth': auth3d 
                })
            return render_to_response(template, context_instance=ctx)

    return secure3d_form_handler(controller)

def secure3d_form_handler(controller):
    """At the confirmation step, protx may ask for a secure3d authentication.  This method
    catches that, and if so, sends to that step, otherwise the form as normal"""
    
    if controller.processorReasonCode == '3DAUTH':
        log.debug('caught secure 3D request for order #%i, putting 3D into session as %s', 
            controller.order.id, controller.processorReasonCode)
            
        redirectUrl = controller.lookup_url('satchmo_checkout-secure3d')
        controller.processor.response['TermUrl'] = redirectUrl
        controller.request.session['3D'] = controller.processorReasonCode
        return http.HttpResponseRedirect(redirectUrl)
    
    return controller.onForm(controller)



# -*- coding: utf-8 -*-
# IMPORTANT! This encoding (charset) setting MUST be correct! If you live in a
# western country and you don't know that you use utf-8, you probably want to
# use iso-8859-1 (or some other iso charset). If you use utf-8 (a Unicode
# encoding) you MUST use: coding: utf-8
# That setting must match the encoding your editor uses when you modify the
# settings below. If it does not, special non-ASCII chars will be wrong.

"""
    MoinMoin - Configuration for a single wiki

    If you run a single wiki only, you can omit the farmconfig.py config
    file and just use wikiconfig.py - it will be used for every request
    we get in that case.

    Note that there are more config options than you'll find in
    the version of this file that is installed by default; see
    the module MoinMoin.config.multiconfig for a full list of names and their
    default values.

    Also, the URL http://moinmo.in/HelpOnConfiguration has
    a list of config options.

    ** Please do not use this file for a wiki farm. Use the sample file
    from the wikifarm directory instead! **
"""

from MoinMoin.config.multiconfig import DefaultConfig


class Config(DefaultConfig):

    # Wiki identity ----------------------------------------------------

    # Site name, used by default for wiki name-logo [Unicode]
    sitename = u'just another Wiki'

    # Wiki logo. You can use an image, text or both. [Unicode]
    # For no logo or text, use '' - the default is to show the sitename.
    # See also url_prefix setting below!
    logo_string = ur'<img src="/media/img/gthc-indigo-alpha2-400x91.png">'

    # name of entry page / front page [Unicode], choose one of those:

    # a) if most wiki content is in a single language
    #page_front_page = u"MyStartingPage"

    # b) if wiki content is maintained in many languages
    page_front_page = u"FrontPage"
    #page_front_page = u"PageD'Accueil"

    # The interwiki name used in interwiki links
    interwikiname = u'just another Wiki'
    # Show the interwiki name (and link it to page_front_page) in the Theme,
    # nice for farm setups or when your logo does not show the wiki's name.
    show_interwiki = 1


    # Critical setup  ---------------------------------------------------

    # Misconfiguration here will render your wiki unusable. Check that

    # all directories are accessible by the web server or moin server.

    # If you encounter problems, try to set data_dir and data_underlay_dir
    # to absolute paths.

    # Where your mutable wiki pages are. You want to make regular
    # backups of this directory.
    data_dir = '/var/www/default_wiki/data/'

    # Where read-only system and help page are. You might want to share
    # this directory between several wikis. When you update MoinMoin,
    # you can safely replace the underlay directory with a new one. This
    # directory is part of MoinMoin distribution, you don't have to
    # backup it.
    data_underlay_dir = '/var/www/default_wiki/underlay/'

    # The URL prefix we use to access the static stuff (img, css, js).
    # NOT touching this is maybe the best way to handle this setting as moin
    # uses a good internal default (something like '/moin_static184' for moin
    # version 1.8.4).
    # For Twisted and standalone server, the default will automatically work.
    # For others, you should make a matching server config (e.g. an Apache
    # Alias definition pointing to the directory with the static stuff).
    url_prefix_static = '/moin_static184'


    # Security ----------------------------------------------------------

    # This is checked by some rather critical and potentially harmful actions,
    # like despam or PackageInstaller action:
    superuser = [u"EtienneRobillard", ]

    # IMPORTANT: grant yourself admin rights! replace YourName with
    # your user name. See HelpOnAccessControlLists for more help.
    # All acl_rights_xxx options must use unicode [Unicode]
    acl_rights_before = u"EtienneRobillard:read,write,delete,revert,admin"

    # The default (ENABLED) password_checker will keep users from choosing too
    # short or too easy passwords. If you don't like this and your site has
    # rather low security requirements, feel free to DISABLE the checker by:
    #password_checker = None # None means "don't do any password strength checks"

    # Link spam protection for public wikis (Uncomment to enable)
    # Needs a reliable internet connection.
    from MoinMoin.security.antispam import SecurityPolicy

    # Textchas ----------------------------------------------------------
    textchas = {
        'en': {u"Enter the code name of notmm 0.3.4?":ur"HappyLady"}
    }
    
    log_reverse_dns_lookups=False


    # Mail --------------------------------------------------------------

    # Configure to enable subscribing to pages (disabled by default)
    # or sending forgotten passwords.

    # SMTP server, e.g. "mail.provider.com" (None to disable mail)
    # mail_smarthost = "gthcfoundation.org"
    mail_smarthost = u"localhost"

    # The return address, e.g u"Jürgen Wiki <noreply@mywiki.org>" [Unicode]
    #mail_from = u"<postmaster@gthcfoundation.org>"
    mail_from = "<postmaster@localhost.localdomain>"

    # "user pwd" if you need to use SMTP AUTH
    #mail_login = ""


    # User interface ----------------------------------------------------

    # Add your wikis important pages at the end. It is not recommended to
    # remove the default links.  Leave room for user links - don't use
    # more than 6 short items.
    # You MUST use Unicode strings here, but you need not use localized
    # page names for system and help pages, those will be used automatically
    # according to the user selected language. [Unicode]
    navi_bar = [
        # If you want to show your page_front_page here:
        #u'%(page_front_page)s',
        u'RecentChanges',
        u'FindPage',
        u'HelpContents',
    ]

    # The default theme anonymous or new users get
    theme_default = 'modernized'


    # Language options --------------------------------------------------

    # See http://moinmo.in/ConfigMarket for configuration in
    # YOUR language that other people contributed.

    # The main wiki language, set the direction of the wiki pages
    language_default = 'en'
    #language_default = 'fr'

    # the following regexes should match the complete name when used in free text
    # the group 'all' shall match all, while the group 'key' shall match the key only
    # e.g. CategoryFoo -> group 'all' ==  CategoryFoo, group 'key' == Foo
    # moin's code will add ^ / $ at beginning / end when needed
    # You must use Unicode strings here [Unicode]
    page_category_regex = ur'(?P<all>Category(?P<key>(?!Template)\S+))'
    page_dict_regex = ur'(?P<all>(?P<key>\S+)Dict)'
    page_group_regex = ur'(?P<all>(?P<key>\S+)Group)'
    page_template_regex = ur'(?P<all>(?P<key>\S+)Template)'

    # Content options ---------------------------------------------------

    # Show users hostnames in RecentChanges
    show_hosts = 0

    # Enable graphical charts, requires gdchart.
    #chart_options = {'width': 600, 'height': 300}




from datetime import datetime
from django.test import TestCase
from l10n.models import Country
from shipping.modules.tieredweight.models import Carrier, TieredWeightException

try:
    from decimal import Decimal
except:
    from django.utils._decimal import Decimal


def make_tiers(zone, prices, expires=None):
    for weight, handling, price in prices:
        zone.tiers.create(
            min_weight=Decimal(weight),
            handling=Decimal('%i.00' % handling),
            price=Decimal('%i.00' % price),
            expires=expires
        )


class TieredWeightTest(TestCase):
    def setUp(self):
        self.carrier = Carrier.objects.create(name='pricing', active=True)
        self.zone = self.carrier.zones.create(name='zone 1')
        self.zone.tiers.create(
            min_weight=Decimal('1'),
            handling=Decimal('10.00'),
            price=Decimal('1.00'),
        )


    def testBase(self):
        self.assertEqual(self.zone.cost(1), Decimal('11.00'))
        self.assertRaises(TieredWeightException, self.zone.cost, 4)

        
    def testTwoPrices(self):
        self.zone.tiers.create(
            min_weight=Decimal('10.00'),
            handling=Decimal('10.00'),
            price=Decimal('2.00'),
        )
        self.assertEqual(self.zone.cost(1), Decimal('11.00'))
        self.assertEqual(self.zone.cost(9), Decimal('12.00'))
        self.assertEqual(self.zone.cost(10), Decimal('12.00'))
        self.assertRaises(TieredWeightException, self.zone.cost, 100)


class TieredWeightExpiringTest(TestCase):
    def setUp(self):
        self.carrier = Carrier.objects.create(name='pricing', active=True)
        self.zone = self.carrier.zones.create(name='zone 1')
        base_prices = (
            (1, 10, 0),
            (20, 20, 1),
            (30, 30, 2),
            (40, 40, 1)
        )
        make_tiers(self.zone, base_prices)


    def testExpired(self):
        expires = datetime(2000, 1, 1)
        sale_prices = (
            (1, 1, 0),
            (20, 2, 1),
            (30, 3, 1),
            (40, 4, 1)
        )
        make_tiers(self.zone, sale_prices, expires=expires)

        self.assertEqual(self.zone.cost(1), Decimal('10.00'))
        self.assertEqual(self.zone.cost(20), Decimal('21.00'))
        self.assertEqual(self.zone.cost(30), Decimal('32.00'))
        self.assertEqual(self.zone.cost(40), Decimal('41.00'))


    def testNotExpired(self):
        now = datetime.now()
        nextyear = datetime(now.year+1, now.month, now.day)
        sale_prices = (
            (1, 1, 0),
            (20, 2, 1),
            (30, 3, 1),
            (40, 4, 1)
        )
        make_tiers(self.zone, sale_prices, expires=nextyear)

        self.assertEqual(self.zone.cost(1), Decimal('1.00'))
        self.assertEqual(self.zone.cost(10), Decimal('3.00'))
        self.assertEqual(self.zone.cost(20), Decimal('3.00'))
        self.assertEqual(self.zone.cost(30), Decimal('4.00'))
        self.assertEqual(self.zone.cost(40), Decimal('5.00'))


class TieredWeightCountryTest(TestCase):
    def setUp(self):
        self.country1 = Country.objects.create(
            iso2_code='mc',
            name='MYCOUNTRY',
            printable_name='MyCountry',
            iso3_code='mgc',
            continent='EU'
        )
        self.country2 = Country.objects.create(
            iso2_code='nc',
            name='NOTMYCOUNTRY',
            printable_name='NotMyCountry',
            iso3_code='nmc',
            continent='EU'
        )
        self.carrier = Carrier.objects.create(name='pricing', active=True)
        self.zone1 = self.carrier.zones.create(name='zone 1')
        self.zone2 = self.carrier.zones.create(name='zone 2')

        self.zone1.countries.add(self.country1)
        self.carrier.default_zone = self.zone2


    def testDefault(self):
        zone = self.carrier.get_zone(self.country1)
        self.assertEqual(zone, self.zone1)


    def testCountry(self):
        zone = self.carrier.get_zone(self.country2)
        self.assertEqual(zone, self.zone2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2010 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
# <LICENSE=ISC>
"""A class-based URL resolving module for Django.

* RegexURLMap:
    Provides a easy-to-use API to build logical groups of callbacks 
    to URIs.
* RegexURLMapException: 
    Reserved for future uses; Exception handling of ``RegexURLMap`` instances.
* include, patterns, and url functions are provided as helpers while upgrading from Django to RegexURLMap.

"""

from importlib import import_module
import re
import logging

from .django_compat import RegexURLPattern, RegexURLResolver

log = logging.getLogger(__name__)

__all__ = ['RegexURLMapException', 'RegexURLMap', 'url', 'include', 'patterns']

include = lambda urlconf: [urlconf]

def patterns(prefix, *args):
    pattern_list = []
    for t in args:
        if isinstance(t, (list, tuple)):
            t = url(prefix=prefix, *t)
        elif isinstance(t, RegexURLPattern):
            t.add_prefix(prefix)
        pattern_list.append(t)
    return pattern_list

def url(regex, view, kwargs={}, name=None, namespace=None, prefix=''):
    if isinstance(view, (tuple, list)):
        # For include(...) processing.
        return RegexURLResolver(regex, view[0], kwargs, namespace=namespace)
    else:
        if isinstance(view, str):
            if not view:
                raise RegexURLMapException('Empty URL pattern view name not permitted (for pattern %r)' % regex)
            if prefix:
                view = prefix + '.' + view
        return RegexURLPattern(regex, view, kwargs, name=name)

class RegexURLMapException(Exception):
    """
    Generic error manipulating a ``RegexURLMap`` object.
    
    """

class RegexURLMap(object):
    """
    Add or create a list of available ``RegexURLPattern`` instances.
 
    Usage ::

    .. coding: python

        >>> from notmm.controllers.routing import RegexURLMap
        >>> urlpatterns = RegexURLMap(label='default')
        >>> print repr(urlpatterns)
        <RegexURLMap: 'default'> 
        >>> urlpatterns.add_routes('myapp.views', ...) # default
        >>> urlpatterns.include(foo.bar.urls) # include urls from another module
    """

    def __init__(self, label='yadayada', verbosity=1):
        self.routes = []
        self.label = label # Label for this urlmap object
        self.debug = (verbosity >= 2)
        self.namespace = label
    
    def __len__(self):
        # Return the number of routes available for this urlmapper
        return len(self.routes)

    def __repr__(self):
        return "<RegexURLMap: %r>" % self.label
    
    def add_routes(self, callback_prefix='', *args):
        """
        Adds a sequence of ``RegexURLPattern`` instances for routing
        valid HTTP requests to a WSGI callback function.

        Unlike the original ``include`` hook provided in Django, this 
        method can be used to create logical groups of routes within the
        same urls.py file. 
        
        """
        self.callback_prefix = callback_prefix
        
        for t in args:
            if hasattr(t, 'name'):
                self.makeroute(t, app_label=t.name)
            else:
                self.makeroute(t)

    def makeroute(self, t, app_label=None):
        """Connect a wsgi app to a URI. Invoked by ``add_routes`` to
        flattenize all urls.py into a immutable list of URIs."""
        route = None
        if app_label and self.debug:
            log.debug("New route: %s" % app_label)
            
        if isinstance(t, (list, tuple)):
            #assert len(t) >= 2, t

            if len(t) == 2:
                regex, view = t
                kwargs = {}
            elif len(t) == 3:
                regex, view, kwargs = t
            elif len(t) == 4:
                regex, view, kwargs, app_label = t
            else:
                #not implemented
                raise
            route = url(regex, view, kwargs=kwargs, name=app_label, 
                prefix=self.callback_prefix, namespace=self.namespace)
            #assert isinstance(route, RegexURLPattern), type(route)

        elif isinstance(t, RegexURLPattern):
            if app_label is not None:
                assert (t.name == app_label), (t.name, app_label)
            #    #t.name = app_label
                if self.debug:
                    log.debug("Registering view: %s" % t.name)
                    if hasattr(t, '_callback_str'):
                        log.debug("callback_str: %s" % t._callback_str)
            route = t
        elif isinstance(t, RegexURLResolver):
            for route in t.url_patterns:
                #log.debug("Found new route: %s" % route)
                route.add_prefix(self.callback_prefix)
                self.routes.append(route)
            #route.namespace = self.namespace
            #    raise RegexURLMapException("app_label is required here!")
            #if app_label is not None:
            #    log.debug("Setting route name to %s" % self.label)
            #    route.app_name = app_label
        else:
            log.debug("Discarding route: %s" % t)

        if isinstance(route, RegexURLPattern):
            route.add_prefix(self.callback_prefix)
            #assert (route.name == app_label), route.name
        try:
            self.routes.append(route)
        except AttributeError as e:
            raise RegexURLMapException(e)


    def include(self, urlobj, prefix='', callback_prefix='',
        attr_name='urlpatterns', app_label=None):
        """Includes urls found in module ``urlobj``.
        
        """
        #objtype = type(urlobj)
        if isinstance(urlobj, tuple):
            urls = urlobj[0] # handle admin.site.urls
        #elif objtype in (str, basestring):
        else:
            urlconf = import_module(urlobj)
            #log.debug("urlconf=%r"%urlconf)
            urls = getattr(urlconf, attr_name)
        #else:
        #    # not a string type, neither a sequence, so its the default
        #    # behavior (urlconf)
        #    try:
        #        urls = getattr(urlobj, attr_name)
        #    except AttributeError:
        #        #assert attr_name in urlobj.__dict__, 'Not a valid urls.py file.'
        #        raise RegexURLMapException('urlconf missing attribute %r' % attr_name)
        #log.debug(urls)
        self.callback_prefix = callback_prefix
        #self.label = app_label

        log.debug("Processing urlconf: %s"%type(urls))

        for instance in urls:
            if isinstance(instance, RegexURLPattern):
                pattern = prefix + instance.regex.pattern[0:]
                log.debug("Including pattern: %s" % pattern)
                try:
                    instance.__dict__['regex'] = re.compile(pattern, re.UNICODE)
                    if not instance.name:
                        instance.__dict__['name'] = app_label
                    #instance.__dict__['_callback_str'] = app_label
                    #instance.__dict__['_reverse_dict'] = instance.reverse_dict
                except re.error:
                    #   already included pattern, skip and continue parsing
                    #   the remaining patterns.
                    continue
                #self.add_routes(callback_prefix, *urls)
                self.makeroute(instance, app_label=instance.name)
            elif isinstance(instance, RegexURLResolver):
                self.makeroute(instance)
            else:
                #Not implemented
                #print type(instance)
                pass            
    
    def commit(self):
        """Freeze the routes. (cannot add anymore)"""
        self.routes = frozenset([item for item in self.routes])
        #self._commit_flag = True
        if self.debug:
            #print self.routes
            log.debug("Configured routes: %d" % len(self.routes))


    def __iter__(self):
        return iter(self.routes)

    def __next__(self):
        for item in self.routes:
            yield (item)

# default patterns for backward compatibility
# patterns = RegexURLMap()



# -*- coding: utf-8 -*-
"""Provides utilities to help multi-site aware applications.

:Authors:
    - Bruce Kroeze
"""
"""
New BSD License
===============
Copyright (c) 2008, Bruce Kroeze http://coderseye.com

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of SolidSiteSolutions LLC, Zefamily LLC nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
__docformat__="restructuredtext"

from threaded_multihost import threadlocals
from django.conf import settings
from django.contrib.sites.models import Site
from django.db.models.loading import app_cache_ready
import keyedcache
import logging

log = logging.getLogger('threaded_multihost.sites')
_WARNED = {}

class MultihostNotReady(Exception):
    pass

def by_host(host=None, id_only=False):
    """Get the current site by looking at the request stored in the thread.  
    
    Returns the best match found in the `django.contrib.sites` app.  If not
    found, then returns the default set as given in `settings.SITE_ID`
    
    Params:
     - `host`: optional, host to look up
     - `id_only`: if true, then do not retrieve the full site, just the id.
    """
    global _WARNED
    if id_only:
        site = -1
    else:
        site = None
        
    debug_domain = None
    if settings.DEBUG and hasattr(settings, 'MULTIHOST_DEBUG_DOMAIN'):
        raw = settings.MULTIHOST_DEBUG_DOMAIN
        parts = raw.split('=')
        if len(parts) == 2:
            debug_domain = parts[0]
            debug_target = parts[1]
        else:
            debug_domain = raw
            debug_target = 'com'

    if not host:
        request = threadlocals.get_current_request()
        if request:
            host = request.get_host()
        else:
            log.debug('No request')
            site = by_settings(id_only=id_only)
        
    if host:
        if app_cache_ready():
            try:
                site = keyedcache.cache_get('SITE', host=host, id_only=id_only)
            except keyedcache.NotCachedError, nce:
                try:
                    log.debug('looking up site by host: %s', host)
                    site = Site.objects.get(domain=host)
                except Site.DoesNotExist:
                    if host.find(":") > -1:
                        try:
                            # strip the port
                            host = host.split(":")[0]
                            site = Site.objects.get(domain=host)
                        except Site.DoesNotExist:
                            pass
                    if debug_domain and host.endswith(debug_domain):
                        host = host[:-len(debug_domain)] + debug_target
                        log.debug('Using debug domain: %s', host)
                        try:
                            site = Site.objects.get(domain=host)
                        except Site.DoesNotExist:
                            pass

                if site:
                    keyedcache.cache_set(nce.key, value=site)
                    
                    if id_only:
                        site = site.id
                    
                else:
                    if not host in _WARNED:
                        log.warn("Site for '%s' is not configured on this site - add to sites in admin", host)
                        _WARNED[host] = True
                        
                    site = by_settings(id_only=id_only)
                
        else:
            log.debug('app cache not ready')
            site = by_settings(id_only=id_only)
    else:
        pass
        #log.debug('no host')    
            
    # if site is None or site == -1:
    #     raise MultihostNotReady()
            
    #log.debug("current site: %s", site)
    return site
        
def by_request(request=None, id_only=False):
    """Look up a site given an explicit request rather than using the request
    currently in the `threadlocals`.
    
    Params:
     - `request`: optional, the request to use, defaults to the request found in the threadlocals
     - `id_only`: if true, then only the id is returned.
    """
    host = None
    if request and 'HTTP_HOST' in request.META:
        host = request.META['HTTP_HOST']
    return by_host(host=host, id_only=id_only)
        
def by_settings(id_only=False):
    """Get the site according to the SITE_ID in settings.
    
    Params:
     - `id_only`: if true, then only the id is returned.
    """
    #log.debug('from_settings')
    global _WARNED
    if id_only:
        return settings.SITE_ID
        
    try:
        return Site.objects.get(pk=settings.SITE_ID)
    except Exception, e:
        message = e.args[0]
        if message.find("django_site") > 0:
            site = None
            if not 'django_site' in _WARNED:
                log.warn("Error, couldn't find django_site in database, ok if you are in syncdb")
                _WARNED['django_site'] = True
        else:
            raise
            
    return site



from django.db.models import signals
from django.db.models.fields.files import ImageField
from livesettings import config_value, SettingNotSet
from satchmo_utils.thumbnail.utils import remove_model_thumbnails, rename_by_field
from satchmo_utils import normalize_dir
import config
import logging
import os

log = logging.getLogger('thumbnail.fields')

def _delete(sender, instance=None, **kwargs):
    if instance:
        if hasattr(instance.picture,'path') and os.path.isfile(instance.picture.path):
            if os.path.isfile(instance.picture.path):
                remove_model_thumbnails(instance)
        else:
            remove_model_thumbnails(instance)
        
def upload_dir(instance, filename):
    raw = "images/"

    try:
        raw = config_value('PRODUCT', 'IMAGE_DIR')
    except SettingNotSet:
        pass
    except ImportError:
        log.warn("Error getting upload_dir, OK if you are in SyncDB.")
        
    updir = normalize_dir(raw)
    return os.path.join(updir, filename)
        

NOTSET = object()

class ImageWithThumbnailField(ImageField):
    """ ImageField with thumbnail support

        auto_rename: if it is set perform auto rename to
        <class name>-<field name>-<object pk>.<ext>
        on pre_save.
    """

    def __init__(self, verbose_name=None, name=None,
                 width_field=None, height_field=None,
                 auto_rename=NOTSET, name_field=None, 
                 upload_to=upload_dir, **kwargs
                 ):
        self.auto_rename = auto_rename

        self.width_field, self.height_field = width_field, height_field
        if upload_to == "__DYNAMIC__":
            upload_to = upload_dir
        super(ImageWithThumbnailField, self).__init__(verbose_name, name,
                                                      width_field, height_field,
                                                      upload_to=upload_to,
                                                      **kwargs)
        self.name_field = name_field
        self.auto_rename = auto_rename
        
    def _save_rename(self, instance, **kwargs):
        if hasattr(self, '_renaming') and self._renaming:
            return
        if self.auto_rename == NOTSET:
            try:
                self.auto_rename = config_value('THUMBNAIL', 'RENAME_IMAGES')
            except SettingNotSet:
                self.auto_rename = False
        
        image = getattr(instance, self.attname)
        if image and self.auto_rename:
            if self.name_field:
                field = getattr(instance, self.name_field)
                image = rename_by_field(image.path, '%s-%s-%s' \
                                        % (instance.__class__.__name__,
                                           self.name,
                                           field
                                           )
                                        )
            else:
                # XXX this needs testing, maybe it can generate too long image names (max is 100)
                image = rename_by_field(image.path, '%s-%s-%s' \
                                        % (instance.__class__.__name__,
                                           self.name,
                                           instance._get_pk_val()
                                           )
                                        )
            setattr(instance, self.attname, image)
            self._renaming = True
            instance.save()
            self._renaming = False

    def contribute_to_class(self, cls, name):
        super(ImageWithThumbnailField, self).contribute_to_class(cls, name)
        signals.pre_delete.connect(_delete, sender=cls)
        signals.post_save.connect(self._save_rename, sender=cls)




#!/usr/bin/env python

from test_support import unittest

# schevo
from schevo.database2 import Database 
from test_xdserver import XdserverProxy
#from model import ActorManager, MovieCastingManager, db
from model import db

class TutorialTestCase(unittest.TestCase):

    def setUp(self):
        super(TutorialTestCase, self).setUp()
        self.db = db

        # populate the database with sample data?
        # self.db.populate()

    def tearDown(self):
        #self.db.close()
        pass

    def runTest(self):
        pass

    def test_open_database(self):

        db = self.db
        #self.assertEqual(repr(db), "<Database u'Schevo Database' :: V 1>")
        
        # now lets make some introspection on db 
        self.assertEqual(isinstance(db, XdserverProxy), True, type(db))
        self.assertEqual(db.version, 1)
        self.assertEqual(db.format==2, True, db.format)

    #def test_find_keanu_reeves(self):
    #    import schevo.query as Q
    #    Person = ActorManager.objects.get(name=u"Keanu Reeves")
    #    #self.failUnlessEqual(isinstance(Person, db.Actor.__class__), True, type(Person))
    #    movie_list = MovieCastingManager.objects.find(actor=Person)
    #    self.assertEqual(isinstance(movie_list, Q.ResultsList), True)




#!/usr/bin/env python
#from django.utils.translation.trans_real import DjangoTranslation
from django.middleware.locale            import LocaleMiddleware
from notmm.controllers.wsgi              import WSGIController
from notmm.utils.wsgilib                 import HTTPResponse
from notmm.utils.django_settings import LazySettings

from test_support                        import *

import gettext

class Translation(object):
    """
    The purpose of this class is to store the actual translation function upon
    receiving the first call to that function. After this is done, changes to
    ``USE_I18N`` will have no effect to which function is served upon request. If
    your tests rely on changing ``USE_I18N``, you can delete all the functions
    from _trans.__dict__.

    Note that storing the function with setattr will have a noticeable
    performance effect, as access to the function goes the normal path,
    instead of using __getattr__.
    """

    def __init__(self, app_label):
        self.app_label = app_label

    def __getattr__(self, real_name):


        if settings.USE_I18N:
            from django.utils.translation import trans_real as trans
            # Uncomment to enable new LOCALE_PATHS behavior 
            #project = import_module(self.app_label)
            #project_locale_path = path.normpath(
            #    path.join(path.dirname(project.__file__), 'locale'))
            #
            # Make sure the project's locale dir isn't in LOCALE_PATHS
            # normalized_locale_paths = [path.normpath(locale_path)
            # for locale_path in settings.LOCALE_PATHS]
            #    if (path.isdir(project_locale_path) and
            #    not project_locale_path in normalized_locale_paths):
            #        warnings.warn("Translations in the project directory "
            #                      "aren't supported anymore. Use the "
            #                      "LOCALE_PATHS setting instead.",
            #                      PendingDeprecationWarning)
        else:
            from django.utils.translation import trans_null as trans
        setattr(self, real_name, getattr(trans, real_name))
        return getattr(trans, real_name)

class I18NLocaleManager(Translation):
    pass

class I18NLocaleManagerTestCase(unittest.TestCase):
    
    LocaleManager = I18NLocaleManager

    def setUp(self):
        pass

    def test_ugettext_lazy(self):
        locale = self.LocaleManager('sandbox')
        _ = locale.gettext
        #print _('hello world')
        self.assertEqual(callable(_), True)
        self.assertRaises(TypeError, _, 1234, 'expected a string value!')
        t1 = _('hello world')
        self.assertEqual(t1, unicode('hello world'), 'expected a unicode type object!')



#!/usr/bin/env python
# Copyright (C) 2007-2013 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
#
# <LICENSE=APACHEV2>
"""
Name
====

notmm.utils.testlib - Test utilities for enhanced unittest integration
in Python.

Description
===========

This module supports recursive packages loading and testing with
unittest module. Require Python 2.7.3+. Untested with Python 3.x.

Usage
=====

Run test scripts in package ``foo``:

 % run.py -C ``foo``

Basic configuration
===================

In your development.ini, add a section name "testrunner" like so:

 [testrunner]

 collections=foo bar

Then run the following:

 % mkdir -p {foo,bar}/__init__.py

Launch the test runner with:

 % ./run.py -C foo bar 

Examples
========

See ``tests/run.py`` for a example test runner script.

"""

from .collection2 import (
    TestCollection, get_test_modules, CollectionError)
from .base import SimpleTestRunner

__all__ = ('collection2', 'base', 'TestCollection', 'get_test_modules',
    'CollectionError', 'SimpleTestRunner')

#



from django import template

register = template.Library()

def product_upsell(product):
    """
    Display the list of products that are upsell candidates for currently viewed product.
    """
    goals = None
    if product.upselltargets.count() > 0:
        goals = product.upselltargets.all()
        
    return { 'goals' : goals }
register.inclusion_tag("upsell/product_upsell.html", takes_context=False)(product_upsell)


# encoding: utf-8
from south.db import db
from south.v2 import SchemaMigration

class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_table('product_customproduct', 'custom_customproduct')
        db.rename_table('product_customproduct_option_group', 'custom_customproduct_option_group')
        db.rename_table('product_customtextfield', 'custom_customtextfield')
        db.rename_table('product_customtextfieldtranslation', 'custom_customtextfieldtranslation')

    def backwards(self, orm):
        db.rename_table('custom_customproduct', 'product_customproduct')
        db.rename_table('custom_customproduct_option_group', 'product_customproduct_option_group')
        db.rename_table('custom_customtextfield', 'product_customtextfield')
        db.rename_table('custom_customtextfieldtranslation', 'product_customtextfieldtranslation')

    models = {
        'custom.customproduct': {
            'Meta': {'object_name': 'CustomProduct'},
            'deferred_shipping': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'downpayment': ('django.db.models.fields.IntegerField', [], {'default': '20'}),
            'option_group': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.OptionGroup']", 'blank': 'True'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'custom.customtextfield': {
            'Meta': {'unique_together': "(('slug', 'products'),)", 'object_name': 'CustomTextField'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'price_change': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '14', 'decimal_places': '6', 'blank': 'True'}),
            'products': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'custom_text_fields'", 'to': "orm['custom.CustomProduct']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'custom.customtextfieldtranslation': {
            'Meta': {'unique_together': "(('customtextfield', 'languagecode', 'version'),)", 'object_name': 'CustomTextFieldTranslation'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'customtextfield': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'to': "orm['custom.CustomTextField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'languagecode': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'version': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'product.category': {
            'Meta': {'unique_together': "(('site', 'slug'),)", 'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'child'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'related_categories': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_categories'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'product.optiongroup': {
            'Meta': {'object_name': 'OptionGroup'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        },
        'product.product': {
            'Meta': {'unique_together': "(('site', 'sku'), ('site', 'slug'))", 'object_name': 'Product'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'also_purchased': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'also_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'category': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Category']", 'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'height': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'height_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'length': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'length_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'related_items': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'shipclass': ('django.db.models.fields.CharField', [], {'default': "'DEFAULT'", 'max_length': '10'}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sku': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'taxClass': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.TaxClass']", 'null': 'True', 'blank': 'True'}),
            'taxable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'total_sold': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'weight': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'weight_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'width': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'width_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'})
        },
        'product.taxclass': {
            'Meta': {'object_name': 'TaxClass'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['custom']



# Copyright (C) 2005 Johan Dahlin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import sys
PY3K = sys.version_info[0] == 3
import gi
import gobject
import gtk
from gtk import gdk

def enumfromstring(value_name, pspec=None, enum=None):
    if not value_name:
        return 0

    try:
        value = int(value_name)
    except ValueError:
        if pspec:
            enum_class = pspec.enum_class
            if enum_class is None:
                return 0
        elif enum:
            enum_class = enum
        else:
            raise ValueError("Need pspec or enm")

        for value, enum in list(enum_class.__enum_values__.items()):
            if value_name in (enum.value_name, enum.value_nick):
                return value

    raise ValueError("Invalid enum value: %r" % value_name)

def flagsfromstring(value_name, pspec=None, flags=None):
    if not value_name:
        return 0

    try:
        return int(value_name)
    except ValueError:
        pass

    value_names = [v.strip() for v in value_name.split('|')]

    if pspec:
        flags_class = pspec.flags_class
    elif flags:
        flags_class = flags
    else:
        raise ValueError("need pspec or flags")

    flag_values = flags_class.__flags_values__
    new_value = 0
    for mask, flag in list(flag_values.items()):
        if (flag.value_names[0] in value_names or
            flag.value_nicks[0] in value_names):
            new_value |= mask

    return new_value

def valuefromstringsimpletypes(gtype, string):
    if gtype in (gobject.TYPE_CHAR, gobject.TYPE_UCHAR):
        value = string[0]
    elif gtype == gobject.TYPE_BOOLEAN:
        value = str2bool(string)
    elif gtype in (gobject.TYPE_INT, gobject.TYPE_UINT):
        value = int(string)
    elif gtype in (gobject.TYPE_LONG, gobject.TYPE_ULONG):
        value = int(string)
    elif gtype in (gobject.TYPE_FLOAT, gobject.TYPE_DOUBLE):
        value = float(string)
    elif gtype == gobject.TYPE_STRING:
        value = string
    elif gobject.type_is_a(gtype, gobject.TYPE_PYOBJECT):
        value = string
    elif gobject.type_is_a(gtype, gobject.GBoxed):
        print ('TODO: boxed')
        value = None
    else:
        raise TypeError("type %r is unknown" % gtype)

    return value

def str2bool(value):
    return value[0].lower() in ('t', 'y') or value == '1'

def get_child_pspec_from_name(gtype, name):
    if gtk.pygtk_version < (2, 10, 0):
        pspecs = gtk.container_class_list_child_properties(gtype)
    else:
        pspecs = gtype.list_child_properties()

    for pspec in pspecs:
        if pspec.name == name:
            return pspec

def get_pspec_from_name(gtype, name):
    for pspec in gobject.list_properties(gtype):
        if pspec.name == name:
            return pspec

class AdapterMeta(type):
    def __new__(mcs, name, bases, dict):
        t = type.__new__(mcs, name, bases, dict)
        t.add_adapter()
        return t

class IObjectAdapter:
    """This interface is used by the loader to build
    a custom object.

    object_type is a GType representing which type we can construct
    """
    object_type = None

    def construct(self, name, properties):
        """constructs a new type of type gtype
        name:  string representing the type name
        gtype: gtype of the object to be constructed
        properties: construct properties
        """
        pass

    def add(self, gobj, child, properties, type):
        """Adds a child to gobj with properties"""
        pass

    def get_properties(self, gtype, obj_id, properties):
        pass

    def set_properties(self, gobj, properties):
        pass

    def prop_set_NAME(self, widget, value_string):
        """
        Sets the property NAME for widget, note that you have to convert
        from a string manully"""
        pass

    def find_internal_child(self, gobj, name):
        """Returns an internal child"""

    def get_internal_child_name(self, gobj, child):
        """
        Returns the name of a widget, as an internal child or None
        if it cannot be found
        """

class Adapter(object, metaclass=AdapterMeta):
    _adapters = []
    _properties = {}

    def __init__(self, build):
        self._build = build

    def get_raw_property(self, object, name):
        object_id = object.get_data('gazpacho::object-id')
        if not object_id in self._properties:
            raise KeyError(object_id)
        return self._properties[object_id][name]

    def add_adapter(cls):
        if cls.__name__ != 'Adapter':
            cls._adapters.append(cls)
    add_adapter = classmethod(add_adapter)

    def get_adapters(cls):
        return cls._adapters
    get_adapters = classmethod(get_adapters)

class AdapterRegistry:
    def __init__(self):
        # GObject typename -> adapter instance
        self._adapters = {}

        for adapter in Adapter.get_adapters():
            self.register_adapter(adapter)

    def _add_adapter(self, name, adapter):
        self._adapters[name] = adapter

    def register_adapter(self, adapter):
        adict = adapter.__dict__
        # Do not register abstract adapters
        if 'object_type' in adict:
            object_type = adict.get('object_type')
            if type(object_type) != tuple:
                object_type = object_type,

            for klass in object_type:
                self._add_adapter(gobject.type_name(klass), adapter)
        elif 'object_name' in adict:
            self._add_adapter(adict.get('object_name'), adapter)

    def get_adapter(self, gobj, type_name=None, build=None):
        # Name is optional, for performance reasons we want
        # to send it as often as we can, normally it's already
        # known at the callsite.
        if type_name == None:
            type_name = gobject.type_name(gobj)
        orig_name = type_name

        while True:
            adapter = self._adapters.get(type_name)
            if adapter:
                # Store a reference for the adapter, to make the
                # next lookup faster
                self._adapters[orig_name] = adapter
                return adapter(build)

            gobj = gobject.type_parent(gobj)
            type_name = gobject.type_name(gobj)

class GObjectAdapter(Adapter):
    object_type = gobject.GObject
    child_names = []
    def construct(self, name, gtype, properties):
        # Due to a bug in gobject.new() we only send in construct
        # only properties here, the rest are set normally
        gobj = gobject.new(gtype, **properties)
        return gobj

    def get_properties(self, gtype, obj_id, properties):
        return self._getproperties(gtype, obj_id, properties)

    def set_properties(self, gobj, properties):
        prop_names = []
        for name, value in properties:
            #print '%s.%s = %r' % (gobject.type_name(gobj), name, value)
            func_name = 'prop_set_%s' % name.replace('-', '_')
            func = getattr(self, func_name, None)
            if func:
                func(gobj, value)
            else:
                gobj.set_property(name, value)

            prop_names.append(name)

        #gobj.set_data('gobject.changed_properties', prop_names)
        setattr(gobj, 'changed_properties', prop_names)

    def _getproperties(self, gtype, obj_id, properties, child=False):
        if child:
            get_pspec = get_child_pspec_from_name
        else:
            get_pspec = get_pspec_from_name

        propdict = self._properties
        if obj_id not in propdict:
            props = propdict[obj_id] = {}
        else:
            props = propdict[obj_id]

        construct = {}
        normal = []
        for prop in properties:
            propname = prop.name.replace('_', '-')
            #full = '%s::%s' % (gobject.type_name(gtype), propname)
            props[prop.name] = prop.data
            if hasattr(self, 'prop_set_%s' % prop.name):
                normal.append((prop.name, prop.data))
                continue

            pspec = get_pspec(gtype, propname)
            if not pspec:
                print(('Unknown property: %s:%s (id %s)' %
                       (gobject.type_name(gtype),
                        prop.name,
                        obj_id)))
                continue

            try:
                value = self._valuefromstring(obj_id, pspec, prop.data)
            except ValueError:
                print(("Convertion failed for %s:%s (id %s), "
                       "expected %s but found %r" %
                       (gobject.type_name(gtype),
                        prop.name,
                        obj_id,
                        gobject.type_name(pspec.value_type),
                        prop.data)))
                continue

            # Could not
            if value is None:
                continue

            if pspec.flags & gobject.PARAM_CONSTRUCT_ONLY != 0:
                construct[pspec.name] = value
            else:
                normal.append((pspec.name, value))

        if child:
            assert not construct
            return normal

        return construct, normal

    def _valuefromstring(self, obj_id, pspec, string):
        # This is almost a 1:1 copy of glade_xml_set_value_from_string from
        # libglade.
        prop_type = pspec.value_type

        if (prop_type in (gobject.TYPE_INT, gobject.TYPE_UINT)
            and gobject.type_name(pspec) == 'GParamUnichar'):
            value = str(string and string[0] or "")
            return value

        try:
            value = valuefromstringsimpletypes(prop_type, string)
        except TypeError:
            pass # it's ok if we fail so far, we'll try other types
        else:
            return value

        if gobject.type_is_a(prop_type, gobject.TYPE_ENUM):
            value = enumfromstring(string, pspec)
        elif gobject.type_is_a(prop_type, gobject.TYPE_FLAGS):
            value = flagsfromstring(string, pspec)
        elif gobject.type_is_a(prop_type, gobject.GObject):
            if gobject.type_is_a(prop_type, gtk.Adjustment):
                value = gtk.Adjustment(0, 0, 100, 1, 10, 10)
                (value.value, value.lower, value.upper,
                 value.step_increment, value.page_increment,
                 value.page_size) = list(map(float, string.split(' ')))
            elif gobject.type_is_a(prop_type, gdk.Pixbuf):
                filename = self._build.find_resource(string)
                value = None
                if filename:
                    # XXX: Handle GError exceptions.
                    value = gdk.pixbuf_new_from_file(filename);
            elif (gobject.type_is_a(gobject.GObject, prop_type) or
                  gobject.type_is_a(prop_type, gobject.GObject)):
                value = self._build.get_widget(string)
                if value is None:
                    self._build.add_delayed_property(obj_id, pspec, string)
            else:
                value = None
        else:
            raise AssertionError("type %r is unknown" % prop_type)

        return value

    def add(self, parent, child, properties, type):
        raise NotImplementedError

    def find_internal_child(self, gobj, name):
        if name in self.child_names:
            return getattr(gobj, name)

    def get_internal_child_name(self, gobj, child):
        for child_name in self.child_names:
            if getattr(gobj, child_name) == child:
                return child_name

class UIManagerAdapter(GObjectAdapter):
    object_type = gtk.UIManager
    def add(self, parent, child, properties, type):
        parent.insert_action_group(child, 0)

class WidgetAdapter(GObjectAdapter):
    object_type = gtk.Widget
    def construct(self, name, gtype, properties):
        widget = GObjectAdapter.construct(self, name, gtype, properties)
        widget.set_name(name)
        return widget

    def prop_set_has_default(self, widget, value):
        value = str2bool(value)
        if value:
            self._build._default_widget = widget

    def prop_set_tooltip(self, widget, value):
        if isinstance(widget, gtk.ToolItem):
            # XXX: factor to separate Adapter
            widget.set_tooltip(self._build._tooltips, value, None)
        else:
            self._build._tooltips.set_tip(widget, value, None)

    def prop_set_visible(self, widget, value):
        value = str2bool(value)
        setattr(widget,'gazpacho::visible', value)
        widget.set_property('visible', value)

    def prop_set_has_focus(self, widget, value):
        value = str2bool(value)
        if value:
            self._build._focus_widget = widget
        #widget.set_data('gazpacho::has-focus', value)

    def prop_set_is_focus(self, widget, value):
        value = str2bool(value)
        #widget.set_data('gazpacho::is-focus', value)

    def prop_set_sizegroup(self, widget, value):
        widget.set_data('gazpacho::sizegroup', value)

    def prop_set_response_id(self, widget, value):
        widget.set_data('gazpacho::response-id', int(value))

class PythonWidgetAdapter(WidgetAdapter):
    def construct(self, name, gtype, properties):
        obj = self.object_type()
        obj.set_name(name)
        return obj

class ContainerAdapter(WidgetAdapter):
    object_type = gtk.Container
    def add(self, container, child, properties, type):
        if not self._add_to_dialog(container, child):
            container.add(child)

        self._set_child_properties(container, child, properties)

    def _add_to_dialog(self, parent, widget):
        if hasattr(widget, 'responseid'):
            dialog = None
            while parent:
                if isinstance(parent, gtk.Dialog):
                    dialog = parent
                    break
                parent = parent.parent

            if dialog:
                # FIXME: Limit to action_area
                dialog.add_action_widget(widget, response_id)

                # Reset, the real properties will be set afterwards
                widget.parent.set_child_packing(widget, True, True, 0,
                                                gtk.PACK_START)
                return True
        return False

    def _set_child_properties(self, container, child, properties):
        properties = self._getproperties(type(container),
                                         child.get_name(),
                                         properties,
                                         child=True)

        for name, value in properties:
            #print '%s.%s = %r (of %s)' % (child.get_name(), name, value,
            #                              gobj.get_name())
            container.child_set_property(child, name, value)

class ActionGroupAdapter(GObjectAdapter):
    object_type = gtk.ActionGroup
    def construct(self, name, gtype, properties):
        if not 'name' in properties:
            properties['name'] = name
        return GObjectAdapter.construct(self, name, gtype, properties)

    def add(self, parent, child, properties, type):
        accel_key = getattr(child, 'accel_key', None)
        if accel_key:
            accel_path = "<Actions>/%s/%s" % (parent.get_property('name'),
                                              child.get_name())
            accel_mod = getattr(child, 'accel_mod')
            gtk.AccelMap.add_entry(accel_path, accel_key, accel_mod)
            child.set_accel_path(accel_path)
            child.set_accel_group(self._build.ensure_accel())
        parent.add_action(child)

class ActionAdapter(GObjectAdapter):
    object_type = gtk.Action
    def construct(self, name, gtype, properties):
        # Gazpacho doesn't save the name for actions
        # So we have set it manually.
        if not 'name' in properties:
            properties['name'] = name
        return GObjectAdapter.construct(self, name, gtype, properties)

    def _set_accel(self, action, accel_key, accel_mod):
        if not accel_key:
            return

        setattr(action, 'accel_key', accel_key)
        setattr(action, 'accel_mod', accel_mod)

    def prop_set_accelerator(self, action, value):
        if not value:
            return
        self._set_accel(action, *gtk.accelerator_parse(value))

    def prop_set_stock_id(self, action, value):
        action.set_property('stock-id', value)

        # Set the accelerator from the stock item, but only if it's
        # not set already

        if action.get_accel_path():
            return

        # FIXME: Should we print a warning if it cannot be found
        #stock_item = gtk.stock_lookup(value)
        #if stock_item:
        #    self._set_accel(action, stock_item[3], stock_item[2])

    # This is for backwards compatibility
    def prop_set_callback(self, action, value):
        if value:
            self._build.add_signal(action, 'activate', value)

class PixmapAdapter(WidgetAdapter):
    object_type = gtk.Image
    def prop_set_build_insensitive(self, pixmap, value):
        pixmap.set_build_insensitive(str2bool(value))

    def prop_set_filename(self, pixmap, value):
        filename = self._build.find_resource(value);
        if not filename:
            print('No such a file or directory: %s' % value)
            return

        try:
            pb = gdk.pixbuf_new_from_file(filename)
        except gobject.GError as e:
            print('Error loading pixmap: %s' % e.message)
            return
        except TypeError as e:
            print('Error loading pixmap: %s' % e)
            return

        pix, bit = pb.render_pixmap_and_mask(127)
        pixmap.set(pix, bit)

class ProgressAdapter(WidgetAdapter):
    object_type = gtk.ProgressBar
    def prop_set_format(self, progress, value):
        progress.set_format_string(value)

class ButtonAdapter(ContainerAdapter):
    object_type = gtk.Button

class OptionMenuAdapter(ButtonAdapter):
    object_type = gtk.MenuBar
    def add(self, parent, child, properties, type):
        if not isinstance(child, gtk.Menu):
            print(("warning: the child of the option menu '%s' was "
                   "not a GtkMenu"  % (child.get_name())))
            return

        parent.set_menu(child)
        self._set_child_properties(parent, child, properties)

    def prop_set_history(self, optionmenu, value):
        optionmenu.set_history(int(value))

class EntryAdapter(WidgetAdapter):
    object_type = gtk.Entry
    def prop_set_invisible_char(self, entry, value):
        entry.set_invisible_char(value.decode()[0])

class TextViewAdapter(ButtonAdapter):
    object_type = gtk.TextView
    def prop_set_text(self, textview, value):
        buffer = gtk.TextBuffer()
        buffer.set_text(value)
        textview.set_buffer(buffer)

class CalendarAdapter(WidgetAdapter):
    object_type = gtk.Calendar
    def prop_set_display_options(self, calendar, value):
        options = flagsfromstring(value, flags=gtk.CalendarDisplayOptions)
        calendar.display_options(options)

class WindowAdapter(ContainerAdapter):
    object_type = gtk.Window
    def prop_set_wmclass_class(self, window, value):
        window.set_wmclass(value, window.wmclass_class)

    def prop_set_wmclass_name(self, window, value):
        window.set_wmclass(value, window.wmclass_name)

    def prop_set_type_hint(self, window, value):
        setattr(window, 'typehint',
                        enumfromstring(value, enum=gdk.WindowTypeHint))

class NotebookAdapter(ContainerAdapter):
    object_type = gtk.Notebook
    def add(self, notebook, child, properties, type):
        tab_item = False
        for propinfo in properties[:]:
            if (propinfo.name == 'type' and
                propinfo.data == 'tab'):
                tab_item = True
                properties.remove(propinfo)
                break

        if tab_item:
            children = notebook.get_children()
            body = notebook.get_nth_page(len(children) - 1)
            notebook.set_tab_label(body, child)
        else:
            notebook.append_page(child)

        self._set_child_properties(notebook, child, properties)

class ExpanderAdapter(ContainerAdapter):
    object_type = gtk.Expander, gtk.Frame
    def _get_label_item(self, properties):
        for propinfo in properties[:]:
            if (propinfo.name == 'type' and
                propinfo.data == 'label_item'):
                properties.remove(propinfo)
                return True
        return False

    def add(self, expander, child, properties, type):
        if self._build._version == 'libglade':
            label_item = self._get_label_item(properties)
        elif type == 'label_item':
            label_item = True
        else:
            label_item = False

        if label_item:
            expander.set_label_widget(child)
        else:
            expander.add(child)

        self._set_child_properties(expander, child, properties)

class MenuItemAdapter(ContainerAdapter):
    object_type = gtk.MenuItem
    def add(self, menuitem, child, properties, type):
        if isinstance(child, gtk.Menu):
            menuitem.set_submenu(child)
        else:
            print('FIXME: Adding a %s to a %s' % (gobject.type_name(child),
                        gobject.type_name(self.object_type)))
            # GtkWarning: Attempting to add a widget with type GtkImage to a
            # GtkImageMenuItem, but as a GtkBin subclass a GtkImageMenuItem
            # can only contain one widget at a time; it already contains a
            # widget of type GtkAccelLabel'
            #menuitem.add(child)

        self._set_child_properties(menuitem, child, properties)

    def prop_set_label(self, menuitem, value):
        child = menuitem.child

        if not child:
            child = gtk.AccelLabel("")
            child.set_alignment(0.0, 0.5)
            menuitem.add(child)
            child.set_accel_widget(menuitem)
            child.show()

        if isinstance(child, gtk.Label):
            child.set_text(value)

    def prop_set_use_underline(self, menuitem, value):
        child = menuitem.child

        if not child:
            child = gtk.AccelLabel("")
            child.set_alignment(0.0, 0.5)
            menuitem.add(child)
            child.set_accel_widget(menuitem)
            child.show()

        if isinstance(child, gtk.Label):
            child.set_use_underline(str2bool(value))

    def prop_set_use_stock(self, menuitem, value):
        child = menuitem.child

        if not child:
            child = gtk.AccelLabel("")
            child.set_alignment(0.0, 0.5)
            menuitem.add(child)
            child.set_accel_widget(menuitem)
            child.show()

        value = str2bool(value)
        if not isinstance(child, gtk.Label) or not value:
            return

        stock_id = child.get_label()

        retval = gtk.stock_lookup(stock_id)
        if retval:
            _, label, modifier, keyval, _ = retval
            # put in the stock image next to the text.  Done before
            # messing with the label child, so that stock_id doesn't
            # become invalid.
            if isinstance(menuitem, gtk.ImageMenuItem):
                image = gtk.image_new_from_stock(stock_id, gtk.ICON_SIZE_MENU)
                menuitem.set_image(image)
                image.show()

            child.set_text(label)
            child.set_use_underline(True)

            if keyval:
                # This triggers a segfault on exit (pachi.glade), weird
                menuitem.add_accelerator('activate',
                                         self._build.ensure_accel(),
                                         keyval, modifier,
                                         gtk.ACCEL_VISIBLE)
        else:
            print("warning: could not look up stock id '%s'" % stock_id)

class CheckMenuItemAdapter(MenuItemAdapter):
    object_type = gtk.CheckMenuItem
    def prop_set_always_show_toggle(self, check, value):
        check.set_show_toggle(value)

class RadioMenuItemAdapter(MenuItemAdapter):
    object_type = gtk.RadioMenuItem
    def prop_set_group(self, radio, value):
        group = self._build.get_widget(value)
        if not group:
            print("warning: Radio button group %s could not be found" % value)
            return

        if group == radio:
            print("Group is self, skipping.")
            return

        radio.set_group(group.get_group()[0])

class ImageMenuItemAdapter(MenuItemAdapter):
    object_type = gtk.ImageMenuItem
    def find_internal_child(self, menuitem, childname):
        if childname == 'image':
            pl = menuitem.get_image()
            if not pl:
                pl = gtk.Image()
                menuitem.set_image(pl)
            return pl
        return MenuItemAdapter.find_internal_child(self, menuitem, childname)

    def get_internal_child_name(self, parent, child):
        if parent.get_image() == child:
            return 'image'
        return MenuItemAdapter.get_internal_child_name(self, parent, child)

class ToolbarAdapter(ContainerAdapter):
    object_type = gtk.Toolbar
    def prop_set_tooltips(self, toolbar, value):
        toolbar.set_tooltips(str2bool(value))

class StatusbarAdapter(ContainerAdapter):
    object_type = gtk.Statusbar
    def prop_set_has_resize_grip(self, status, value):
        status.set_has_resize_grip(str2bool(value))

#class RulerAdapter(WidgetAdapter):
#    object_type = gtk.Ruler
#    def prop_set_metric(self, ruler, value):
#        ruler.set_metric(enumfromstring(value,
#                                        enum=gtk.MetricType))

class ToolButtonAdapter(ContainerAdapter):
    object_type = gtk.ToolButton
    def prop_set_icon(self, toolbutton, value):
        filename = self._build.find_resource(value)
        if not filename:
            print('No such a file or directory: %s' % value)
            return

        pb = gdk.pixbuf_new_from_file(filename)
        if not pb:
            print("warning: Couldn't find image file: %s" %  value)
            return
        image = gtk.Image()
        image.set_from_pixbuf(pb)
        image.show()
        toolbutton.set_icon_widget(image)

class ToggleToolButtonAdapter(ToolButtonAdapter):
    object_type = gtk.ToggleToolButton
    def prop_set_active(self, toolbutton, value):
        toolbutton.set_active(str2bool(value))

def combobox_set_content_from_string(combobox, value):
    # If the "items" property is set, we create a simple model with just
    # one column of text.
    store = gtk.ListStore(str)
    combobox.set_model(store)

    # GtkComboBoxEntry creates the cell renderer itself, but we have to set
    # the column containing the text.
    if isinstance(combobox, gtk.ComboBoxEntry):
        if combobox.get_text_column() == -1:
            combobox.set_text_column(0)
    else:
        cell = gtk.CellRendererText()
        combobox.pack_start(cell, True)
        combobox.set_attributes(cell, text=0)

    for part in value.split('\n'):
        store.append([part])

        active = combobox.get_data('gazpacho::active')
        if active:
            combobox.set_active(active)

class ComboBoxAdapter(ContainerAdapter):
    # FIXME: Internal child: entry -> get_child()
    object_type = gtk.ComboBox, gtk.ComboBoxEntry
    def prop_set_items(self, combobox, value):
        combobox_set_content_from_string(combobox, value)

    def prop_set_active(self, combobox, value):
        combobox.set_data('gazpacho::active', int(value))

        # We don't need to set the actual property here, it's done in
        # prop_set_items.

# FIXME: PyGTK does not expose vscrollbar and hscrollbar.
#
#class ScrolledWindowAdapter(WindowAdapter):
#    child_names = ['vscrollbar', 'hscrollbar']
#    object_type = gtk.ScrolledWindow
#    def get_internal_child_name(self, parent, child):
#        if parent.vscrollbar == child:
#            return 'vscrollbar'
#        elif parent.hscrollbar == child:
#            return 'hscrollbar'
#        return WindowAdapter.get_internal_child_name(self, parent, child)

class DialogAdapter(WindowAdapter):
    child_names = ['vbox', 'action_area']
    object_type = gtk.Dialog
    def get_internal_child_name(self, parent, child):
        if parent.vbox == child:
            return 'vbox'
        elif parent.action_area == child:
            return 'action_area'
        return WindowAdapter.get_internal_child_name(self, parent, child)

class ColorSelectionDialogAdapter(DialogAdapter):
    child_names = DialogAdapter.child_names + ['ok_button',
                                               'cancel_button',
                                               'help_button',
                                               'colorsel']
    object_type = gtk.ColorSelectionDialog
    def find_internal_child(self, gobj, name):
        if name == 'color_selection':
            return gobj.colorsel

        return DialogAdapter.find_internal_child(self, gobj, name)

    def get_internal_child_name(self, parent, child):
        if parent.colorsel == child:
            return 'color_selection'
        return DialogAdapter.get_internal_child_name(self, parent, child)

class FontSelectionDialogAdapter(DialogAdapter):
    object_type = gtk.FontSelectionDialog
    child_names = DialogAdapter.child_names + ['ok_button',
                                               'cancel_button',
                                               'apply_button',
                                               'fontsel']
    def find_internal_child(self, gobj, name):
        if name == 'font_selection':
            return gobj.fontsel

        return DialogAdapter.find_internal_child(self, gobj, name)

    def get_internal_child_name(self, parent, child):
        if parent.fontsel == child:
            return 'font_selection'
        return DialogAdapter.get_internal_child_name(self, parent, child)

class TreeViewAdapter(ContainerAdapter):
    object_type = gtk.TreeView
    def add(self, treeview, child, properties, type):
        if not isinstance(child, gtk.TreeViewColumn):
            raise TypeError("Children of GtkTreeView must be a "
                            "GtkTreeViewColumns, not %r" % child)

        treeview.append_column(child)

        # Don't chain to Container add since children are not widgets

class CellLayoutAdapter(GObjectAdapter):
    def set_cell_renderer_attributes(self, gobj, cell_renderer, attributes):
        raise NotImplementedError

class TreeViewColumnAdapter(CellLayoutAdapter):
    object_type = gtk.TreeViewColumn

    def construct(self, name, gtype, properties):
        gobj = super(TreeViewColumnAdapter, self).construct(name, gtype,
                                                            properties)
        gobj.name = name
        return gobj

    def add(self, column, child, properties, type):
        if not isinstance(child, gtk.CellRenderer):
            raise TypeError("Children of GtkTreeViewColumn must be a "
                            "GtkCellRenderers, not %r" % child)

        expand = True
        pack_start = True
        for propinfo in properties[:]:
            name = propinfo.name
            if name == 'expand':
                expand = str2bool(propinfo.data)
                properties.remove(propinfo)
            elif name == 'pack_start':
                pack_start = str2bool(propinfo.data)
                properties.remove(propinfo)
            else:
                raise AssertionError("Unknown property for "
                                     "GtkTreeViewColumn: %s" % name)
        if pack_start:
            column.pack_start(child, expand)
        else:
            column.pack_end(child, expand)

    def set_cell_renderer_attributes(self, column, cell_renderer, attributes):
        for name, value in list(attributes.items()):
            column.add_attribute(cell_renderer, name, value)

        # this is ugly but's the only way to retrieve the attributes later on
        cell_renderer.set_data('gazpacho::attributes', attributes)

# Gross hack to make it possible to use FileChooserDialog on win32.
# See bug http://bugzilla.gnome.org/show_bug.cgi?id=314527
class FileChooserDialogHack(gtk.FileChooserDialog):
    def get_children(self):
        return []

class FileChooserDialogAdapter(DialogAdapter):
    gobject.type_register(FileChooserDialogHack)

class ImageAdapter(WidgetAdapter):
    object_type = gtk.Image
    def prop_set_file(self, image, value):
        newvalue = self._build.find_resource(value)
        if newvalue:
            image.set_property('file', newvalue)
        image.set_data('image-file-name', value)

class ListStoreAdapter(GObjectAdapter):
    object_type = gtk.ListStore

    def construct(self, name, gtype, properties):
        gobj = super(ListStoreAdapter, self).construct(name, gtype,
                                                       properties)
        gobj.name = name
        return gobj

    def set_column_types(self, store, columns):
        if columns:
            types = [col.type for col in columns]
            store.set_column_types(*types)

    def fill(self, store, rows):
        types = [store.get_column_type(i) for i in range(store.get_n_columns())]
        for row_info in rows:
            values = []
            for col_info in row_info.cols:
                gtype = types[col_info.id]
                value = valuefromstringsimpletypes(gtype, col_info.data)
                values.append(value)

            store.append(values)

# Global registry
adapter_registry = AdapterRegistry()



from decimal import Decimal
from django.contrib.auth.models import User, Group
from django.test import TestCase
from product.models import Product, Price
from satchmo_ext.tieredpricing.models import *
from threaded_multihost.threadlocals import set_current_user
import keyedcache

class TieredTest(TestCase):
    """Test Tiered Pricing"""
    fixtures = ['l10n-data.yaml','sample-store-data.yaml', 'products.yaml', 'test-config.yaml']

    def setUp(self):
        tieruser = User.objects.create_user('timmy', 'timmy@example.com', '12345')
        stduser = User.objects.create_user('tommy', 'tommy@example.com', '12345')
        tieruser.save()
        stduser.save()
        self.tieruser = tieruser
        self.stduser = stduser
        tiergroup = Group(name="tiertest")
        tiergroup.save()
        tieruser.groups.add(tiergroup)
        tieruser.save()
        self.tier = PricingTier(group=tiergroup, title="Test Tier", discount_percent=Decimal('10.0'))
        self.tier.save()

    def tearDown(self):
        keyedcache.cache_delete()

    def test_simple_tier(self):
        """Check quantity price for a standard product using the default price"""
        product = Product.objects.get(slug='PY-Rocks')
        set_current_user(None)
        self.assertEqual(product.unit_price, Decimal("19.50"))
    
    def test_tiered_user(self):
        """Test that a tiered user gets the tiered price"""
        product = Product.objects.get(slug='PY-Rocks')
        set_current_user(self.tieruser)
        # 10% discount from 19.50        
        self.assertEqual(product.unit_price, Decimal("17.550"))
        
    def test_no_tier_user(self):
        """Check price when user doesn't have a tier"""
        product = Product.objects.get(slug='PY-Rocks')
        set_current_user(self.stduser)
        self.assertEqual(product.unit_price, Decimal("19.50"))
        
    def test_tieredprice(self):
        """Test setting an explicit tieredprice on a product"""
        product = Product.objects.get(slug='PY-Rocks')
        tp = TieredPrice(product=product, pricingtier=self.tier, quantity='1', price=Decimal('10.00'))
        tp.save()
        set_current_user(self.tieruser)
        # should be the new explicit price
        self.assertEqual(product.unit_price, Decimal("10.00"))        

    def test_tieredprice_no_tier_user(self):
        """Test setting an explicit tieredprice on a product, but no tier for user"""
        product = Product.objects.get(slug='PY-Rocks')
        tp = TieredPrice(product=product, pricingtier=self.tier, quantity='1', price=Decimal('5.00'))
        tp.save()
        set_current_user(self.stduser)
        self.assertEqual(product.unit_price, Decimal("19.50"))
    